//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "clip.h"
#include "util.h"
#include "color.h"
#include "xet.h"
#include "procSpooler.h"
#include <libc.h>
#include <unistd.h>

extern Xet            *XET;
extern ProcSpooler    *procSpooler;
extern Color          *currentCmap;

Clip::Clip(Display *d,Widget w)
:Dpy(d),dsp_width(0),dsp_height(0),image(NULL),ncolors(0),width(0),height(0),
 shapeMask(NULL),clipStatus(NORMAL_E),RefCounter(0),drawingArea(w),
 diskFileName(NULL),shapeFileName(NULL),clipName(NULL),IsClone(0)
{
}
Clip::Clip()
:dsp_width(0),dsp_height(0),image(NULL),ncolors(0),width(0),height(0),
 shapeMask(NULL),clipStatus(NORMAL_E),RefCounter(0),
 diskFileName(NULL),shapeFileName(NULL),clipName(NULL),IsClone(0)
{
}

Clip::~Clip(){
}

void Clip::setDisplay(Display *d){
  Dpy = d;
}

void Clip::deallocateString(){
  if(diskFileName)   delete diskFileName;
  if(shapeFileName)  delete shapeFileName;
  if(clipName)       delete clipName;
}

void Clip::SetGC(GC g){
  gc = g;
}

void Clip::incrRefCounter(){ 
  RefCounter++;
}

void Clip::decrRefCounter(){ 
  RefCounter--;
  if(RefCounter < 0)
    RefCounter = 0;
}

int  Clip::getRefCounter() {
  return RefCounter;
}
void  Clip::deleteShapeMask(){
  if(shapeMask)
    XFreePixmap(Dpy,shapeMask);
  shapeMask = NULL;
}

void  Clip::setShapeMask(Pixmap p){
  if(shapeMask)
    XFreePixmap(Dpy,shapeMask);
  shapeMask = p;
}

void  Clip::setDiskFileName(char *s){
  ALLOC_STRCPY(diskFileName,s);
}

void  Clip::setShapeFileName(char *s){
  ALLOC_STRCPY(shapeFileName,s);
}

void  Clip::setClipName(char *s){
  ALLOC_STRCPY(clipName,s);
}

char    *Clip::getClipStatusInString(){
  switch(clipStatus){
  case NORMAL_E:        return "Normal    ";
  case IGNOREMASK_E:    return "No Mask   " ;
  case IGNOREIMAGE_E:   return "No Image  " ;
  case IGNOREBOTH_E:    return "No Display" ;
  }
}

void     Clip::setClipStatus(ClipStatus p){
  clipStatus = p;
}

void  Clip::setNcolors(int n){
  ncolors = n;
}

void Clip::MakeImage(int , int , int , int , int , int ){
}

void Clip::MakeImageAndDisplay(OutputWindow *,int , int , int , int , int , int ){
  // only used by cutImage when a clip is partially off the drawing area
}

void Clip::DisplayImage(OutputWindow *widget,int x, int y, int shapemaskbg){
  DisplayImage(widget,x,y,width,height,shapemaskbg);
}

void Clip::DisplayImage(OutputWindow *w,Pixmap p,int x, int y, int shapemaskbg){
  DisplayImage(w,p,x,y,width,height,shapemaskbg);
}

void Clip::DisplayImage(OutputWindow * widget,int x, int y, int w, int h, int shapemaskbg)
{
  DisplayImage(widget,0,x,y,w,h,shapemaskbg);
}

void Clip::DisplayImage(OutputWindow *widget){
  // display image in the middle of the drawingArea
  int w,h;
  w = ((int) widget->da_width - width) / 2;
  h = ((int) widget->da_height - height) / 2;
  DisplayImage(widget,
	       (w <= 0) ? 0 : w,
	       (h <= 0) ? 0 : h,
	       -1);
}

void Clip::DisplayImage(OutputWindow *d_area,Pixmap pixmap,int x, int y, 
			int w, int h, int shapemaskbg)
{
  dsp_x = x; 
  dsp_y = y;
  dsp_width  = w;
  dsp_height = h;
  if(x >= (int) d_area->da_width || y >= (int) d_area->da_height){
    return;
  }
  if(x+w <= 0 || y+h <= 0){ 
    return;
  }
  if(clipStatus == IGNOREBOTH_E)
    return;
  if(shapeMask && clipStatus != IGNOREMASK_E){
    GC            loc_gc;
    XGCValues     gvals; 
    XImage        *img;
    //Pixmap        partialpix = 0;

    Pixmap pix = XCreatePixmap(d_area->Dpy,d_area->window,
			       dsp_width,dsp_height,8);
    loc_gc = XCreateGC(d_area->Dpy,pix, 0, &gvals);
    if(d_area->backgroundImage){
      img = XSubImage(d_area->backgroundImage,dsp_x,dsp_y,dsp_width,dsp_height);
      XPutImage(d_area->Dpy, pix, loc_gc, img, 0, 0,
		0 , 0 , dsp_width,dsp_height);
      XDestroyImage(img);
    }else{
      XSetForeground(d_area->Dpy,loc_gc,WhitePixel(d_area->Dpy,DefaultScreen(d_area->Dpy)));
      XFillRectangle(d_area->Dpy,pix,loc_gc,0,0,dsp_width,dsp_height);
    }
    if(shapemaskbg != -1){ // means to use the specified bg color
      XSetForeground(d_area->Dpy,loc_gc,shapemaskbg);
      XFillRectangle(d_area->Dpy,pix,loc_gc,0,0,dsp_width,dsp_height);
    }
    XSetClipMask(d_area->Dpy,loc_gc,shapeMask);
    if(clipStatus == IGNOREIMAGE_E){
      Pixmap blackpix = XCreatePixmap(d_area->Dpy,
				      d_area->window,
				      dsp_width,dsp_height,8);
      XSetForeground(d_area->Dpy,loc_gc,BlackPixel(d_area->Dpy,DefaultScreen(d_area->Dpy))); 
      XSetBackground(d_area->Dpy,loc_gc,BlackPixel(d_area->Dpy,DefaultScreen(d_area->Dpy))); 
      XFillRectangle(d_area->Dpy,blackpix,loc_gc,0,0,dsp_width,
		     dsp_height);
      XImage *blackimg = XGetImage(d_area->Dpy,blackpix,0,0,
				   dsp_width,dsp_height,0xff,ZPixmap);
      XPutImage(d_area->Dpy, pix, loc_gc, blackimg, 0, 0,
		0 , 0 , dsp_width,dsp_height);
      XFreePixmap(d_area->Dpy,blackpix);
      XDestroyImage(blackimg);
    }else{
      XPutImage(d_area->Dpy, pix, loc_gc, image, 0, 0,
		0 , 0 , dsp_width,dsp_height);
    }
    if(pixmap){
      XCopyArea(d_area->Dpy,pix,pixmap,gc,0,0,
		dsp_width,dsp_height,dsp_x , dsp_y);
    }else
      XCopyArea(d_area->Dpy,pix,d_area->window,gc,0,0,
		dsp_width,dsp_height,dsp_x , dsp_y);
    XFreePixmap(d_area->Dpy,pix);
    //if(partialpix)
    //  XFreePixmap(d_area->Dpy,partialpix);
    XFreeGC(d_area->Dpy,loc_gc);

  }else{
    if(image && dsp_width > 0 && dsp_height > 0)
      if(pixmap)
	XPutImage(d_area->Dpy, pixmap, gc, image, 0, 0,
		  dsp_x , dsp_y , dsp_width,dsp_height);
      else
	XPutImage(d_area->Dpy, d_area->window,gc,image,0,0,
		  dsp_x , dsp_y , dsp_width,dsp_height); 
  }
}

void Clip::PutPixel(int x,  int y, unsigned long v){
  if(image)
    XPutPixel(image,x,y,v);
}

void Clip::writeOut(){
  char buffer[256];
  if(diskFileName == NULL){
    sprintf(buffer,"%s/%s.ppm",EFFECTIVE_DIRECTORY,clipName);
  }else{
    sprintf(buffer,"%s/%s",EFFECTIVE_DIRECTORY,skipDirectoryPath(diskFileName));
  }
  if(IsClone == 0){      // if it is a clone copy don't write it out
    writeXimageInPPM(Dpy,buffer,image);
    insertStringtoInfo("Wrote ppm file <%s> \n",buffer);
  }
}

void Clip::writeOutMask(){
  char buffer[256];
  if(shapeMask){
    if(shapeFileName == NULL)
      sprintf(buffer,"%s/%s.mask",EFFECTIVE_DIRECTORY,clipName);
    else
      sprintf(buffer,"%s/%s",EFFECTIVE_DIRECTORY,skipDirectoryPath(shapeFileName));
    writeOutMask(buffer);
    insertStringtoInfo("Wrote mask file <%s>\n",buffer);
  }
}

void Clip::writeOutMask(char *fileName){
  if(shapeMask)
    XWriteBitmapFile(Dpy,fileName,shapeMask,width,height,0,0);
}

void Clip::writeOut(char *fileName){
  // if the Ximage alread has a corresponding ppm file on disk
  // then we don't have to write it
  // only write out snap shot and text imgages. 
  // The import images must have its corresponding disk file

  // In this release we still write it out
  writeXimageInPPM(Dpy,fileName,image);
}

//void Clip::Clean() {}
void Clip::buildXimage(){}
void Clip::DrawText(char *,Widget,int,int,int,int,int,int,unsigned long,
		    unsigned long,XImage *){}
void Clip::DrawText(char,Widget,int,int,int,int,int,int,unsigned long,
		    unsigned long,XImage *){}
//void Clip::nextMove(){}

Clip *Clip::dup(){
  return new Clip(*this);
}

void Clip::copyto(Clip *rtn){
  // copy these much to the *rtn
  ALLOC_STRCPY(rtn->shapeFileName,shapeFileName);
  ALLOC_STRCPY(rtn->diskFileName,diskFileName);
  ALLOC_STRCPY(rtn->clipName,clipName);
  rtn->shapeMask = copyMask();
  rtn->image = copyImage();
}

XImage *Clip::copyImage(){
  XImage *img = NULL;
  if(image){
    Pixmap pix = XCreatePixmap(Dpy, XtWindow(drawingArea),width,height,8);
    XPutImage(Dpy,pix,gc,image,0,0,0,0,width,height);
    img =  XGetImage(Dpy,pix,0,0,width,height,0xff,ZPixmap);
    XFreePixmap(Dpy,pix);
  }
  return img;
}

Pixmap Clip::copyMask(){
  Pixmap pix = 0;
  if(shapeMask){
    // I hate to see the following 2 statements are not working.
    // the alternative is to write out shapeMask then read it back in
    //rtn->shapeMask = XCreatePixmap(Dpy, XtWindow(drawingArea), width,height,1);
    //XCopyArea(Dpy,shapeMask,rtn->shapeMask,gc,0,0,width,height,0,0);
    char filebuf[100];
    sprintf(filebuf,"%s/xetmask%d",XET_WORKING_DIRECTORY,getpid());
    writeOutMask(filebuf);
    unsigned int w, h;
    int          x_hot, y_hot;
    if(BitmapOpenFailed != XReadBitmapFile(Dpy,XtWindow(drawingArea),
					   filebuf,
					   &w,&h,&pix,&x_hot,&y_hot))
      { 
	insertStringtoInfo("Can not duplicate shapemask in clip <%s>\n",clipName);
      }
    unlink(filebuf);
  }
  return pix;
}


