//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

// The entire file will be re-written in the next release.
//      Ko-Haw Nieh December, 1992

#include "color.h"
#include <stdio.h>
#include <libc.h>
#include "util.h"
#include "config.h"

// follogng colors are used in the resouce file
static char *xetReservedColor[] = {XET_BLACK,
				   XET_DEEPSKYBLUE,
				   XET_FIREBRICK3,
				   XET_GOLDENROD, 
				   XET_GRAY50,    
				   XET_GRAY80,   
				   XET_NAVY,      
				   XET_SKYBLUE4,  
				   XET_TAN,      
				   XET_WHEAT3,    
				   XET_WHITE,     
				   NULL,            // last one must be NULL
				 };
		       
Color::Color(Display *d, Widget w)
:maxval(1),colorHist(NULL),status(DEFAULT_E),privateCmap(0),
NoOfAvailableCells(0),loadedCmap(0)
{
  Dpy = d;
  defaultShell = w;
  effectiveCmap = defaultCmap = 
    DefaultColormap(Dpy,DefaultScreen(Dpy));
  int rs;
  for(rs = 0;;rs++){ // find out how many reserved colors
    if(xetReservedColor[rs] == NULL)
      break;
  }
  reservedColors = new XColor[rs];
  NoOfReservedColors = rs;
}

Color::~Color(){
  if(defaultCmap)
    XFreeColormap(Dpy,defaultCmap);
  if(privateCmap)
    XFreeColormap(Dpy,privateCmap);
  delete colorHist;
}


void Color::allocateAllAvailableCells(){
  unsigned long plane_mask[1];
  NoOfAvailableCells = 256;
  for(;;){
    if(XAllocColorCells(Dpy,effectiveCmap,False,plane_mask,0,
			availableCells,NoOfAvailableCells))
      break;
    NoOfAvailableCells--; 
    if(NoOfAvailableCells == 0)
      break;
  }
  insertStringtoInfo("Available read/write Color Cells = %d\n",NoOfAvailableCells);
  // put white pixels to all avaliable cells, use color panel to see 
  // waht are those available cells ?
  XColor col;
  col.flags = DoRed | DoGreen | DoBlue ;
  col.red = col.green = col.blue = 65535;
  for(int i = 0;i<NoOfAvailableCells;i++){
    col.pixel = availableCells[i];
    XStoreColor(Dpy,effectiveCmap,&col);
  }
  // free all available cells
  XFreeColors(Dpy,effectiveCmap,availableCells,NoOfAvailableCells,0);
}


int Color::allocateNamedColor(char *s){
  XColor col;
  col.flags = DoRed | DoGreen | DoBlue ;
  col.pixel = 0;
  Status st = XParseColor(Dpy,effectiveCmap,s,&col);
  if(s == 0)
    return -1;
  else{
    Status st1 = XAllocColor(Dpy,effectiveCmap,&col);
    if(st1 == 0){
      insertStringtoInfo("can not allocate color <%s>\n",s);
      return -1;
    }
    return (int) col.pixel;
}
}
void Color::locateReservedColors(){
  int i;
  Status st,st1;
  for(i=0;i<NoOfReservedColors;i++){
    if(xetReservedColor[i] == NULL)
      break;
    reservedColors[i].flags = DoRed | DoGreen | DoBlue ;
    reservedColors[i].pixel = 0;
    st = XParseColor(Dpy,effectiveCmap,xetReservedColor[i],&reservedColors[i]);
    if(st == 0){
      reservedColors[i].pixel = 0;
      insertStringtoInfo("can not parse reserved color <%s>\n",xetReservedColor[i]);
    }else{
      st1 = XAllocColor(Dpy,effectiveCmap,&reservedColors[i]);
      if(st1 == 0)
	insertStringtoInfo("can not allocate reserved color <%s>\n",xetReservedColor[i]);
    }
//    sprintf(buffer,"reserve color:: status = %d, pixvel = %d, (%d,%d,%d)\n",
//	    st, (int)reservedColors[i].pixel,reservedColors[i].red,
//	    reservedColors[i].green,reservedColors[i].blue);
//    insertStringtoInfo(buffer);
  }
}


void Color::setMaxval(int m){
  maxval = 256 / (m+1) ;
}

Colormap Color::getcmap(){
  return effectiveCmap;
}

void  Color::setCmapStatus(CmapStatus c){
  status = c;
  if(status == DEFAULT_E){
    loadedCmap = 0;
    if(defaultCmap)
      effectiveCmap = defaultCmap;
    else
      effectiveCmap = defaultCmap = DefaultColormap(Dpy,DefaultScreen(Dpy));
  }
  if(status == PRIVATE_E){
    if(privateCmap)
      effectiveCmap = privateCmap;
    else{
      loadedCmap = 0;
      effectiveCmap = privateCmap = XCopyColormapAndFree(Dpy,effectiveCmap);
    }
  }
  cleanCellsCanBeFreedArray();
  locateReservedColors();
  allocateAllAvailableCells();
}

CmapStatus Color::getCmapStatus(){
  return status;
}

void Color::Initialize(){
  if(colorHist){
    delete colorHist;
  }
  colorHist = new ColorHist(1000);
}

void Color::setWindowColormap(Widget w){
  if(effectiveCmap)
    XSetWindowColormap(XtDisplay(w),XtWindow(w),effectiveCmap);
}

void Color::loadcmap(FILE *cmap){
  if(privateCmap)
    XFreeColormap(Dpy,privateCmap);
  privateCmap = XCreateColormap(Dpy,XtWindow(defaultShell),
				DefaultVisualOfScreen(DefaultScreenOfDisplay(Dpy)),
				AllocAll);
  int ncolors = 256;
  XColor colorCell[256];
  
  int a,b,c,i,where;
  char buf[255];
  for(i=0;i<256;i++){
    colorCell[i].flags = DoRed | DoGreen | DoBlue;
    colorCell[i].red = 65535;
    colorCell[i].green = 65535;
    colorCell[i].blue = 65535;
    colorCell[i].pixel = i;
  }

  i = 0;
  // each line contains 4 numbers, first 3 are r,g,b the last one is the pixval
  // if the last one appears then we put the r,g,b to that cell.
  while(!feof(cmap) && i < ncolors) {
    fgets(buf,sizeof buf,cmap);
    where = 0;
    sscanf(buf,"%d %d %d %d",&a,&b,&c,&where);
    printf("(%d %d %d) where = %d i = %d\n",a,b,c,where,i);
    if(where == 0 && i < 256){
      colorCell[i].flags = DoRed | DoGreen | DoBlue;
      colorCell[i].red = (unsigned short)   (a << 8);
      colorCell[i].green = (unsigned short) (b << 8);
      colorCell[i].blue = (unsigned short)  (c << 8);
      colorCell[i].pixel = i;
      i++;
    }else{
      if(where < 256){
	colorCell[where].flags = DoRed | DoGreen | DoBlue;
	colorCell[where].red = (unsigned short)   (a << 8);
	colorCell[where].green = (unsigned short) (b << 8);
	colorCell[where].blue = (unsigned short)  (c << 8);
	colorCell[where].pixel = where;
	printf("why not set \n");
      }
    }
  }
  fclose(cmap);

  loadedCmap = 1; // set to 1 so this cmap can not be freed

  int locstatus = XStoreColors(Dpy,privateCmap,colorCell,256);
  if( locstatus == BadColor)
    insertStringtoInfo("bad color \n");
  if( locstatus == BadValue)
    insertStringtoInfo("bad value\n");
  if(status == PRIVATE_E){
    effectiveCmap = privateCmap;
    XSetWindowColormap(Dpy,XtWindow(defaultShell),effectiveCmap);
  }
}

void Color::writeCurrentCmapInPPm(char *filename){
  writeCurrentCmap(filename, 1);
}

void Color::writeCurrentCmapInASCII(char *filename){
  writeCurrentCmap(filename, 2);
}

void Color::writeCurrentCmap(char *filename, int flag){
  int ncolors = 256;
  XColor p[256];
  int i;
  for(i=0;i<ncolors;i++){
    p[i].pixel = i;
    p[i].flags = DoRed | DoGreen | DoBlue;
  }
  XQueryColors(Dpy,effectiveCmap,p,ncolors);
  FILE *fp=fopen(filename,"w"); 
  if(fp){
    if(flag == 1){
      fprintf(fp,"P3\n");
      fprintf(fp,"16 16\n");
      fprintf(fp,"255\n");
    }
    int a,b,c;
    for(i=0;i<ncolors;i++){
      a = (int) ((p[i].red)   >> 8 );
      b = (int) ((p[i].green) >> 8 );
      c = (int) ((p[i].blue)  >> 8);
      fprintf(fp,"%d ",a);
      fprintf(fp,"%d ",b);
      fprintf(fp,"%d \n",c);  
    }
    fprintf(fp,"\n");
    fclose(fp);
  }else
    insertStringtoInfo("can not open file <%s>\n",filename);
}


int Color::registerNewColor(unsigned char r,unsigned char g, unsigned char b){
  if(colorHist->Hinsert(r,g,b) == 0){
    return 0;   // table is full
  }else
    return 1;
}

void Color::buildPrivateCmap(){
  if(privateCmap)
    XFreeColormap(Dpy,privateCmap);
  privateCmap = XCreateColormap(Dpy,XtWindow(defaultShell),
				DefaultVisualOfScreen(DefaultScreenOfDisplay(Dpy)),
				AllocAll);
  int ncolors = 256;
  XColor colorCell[256];
  colorHist->startLooping();
  int i;
  ColorEntry *ce;
  for(i=0;;i++){
    ce = colorHist->nextEntry();
    if(ce == NULL)
      break;
    if(i == 256){
      insertStringtoInfo("Too much colors in the ppm file ignore the rest \n");
      break;
    }
    colorCell[i].red =   (unsigned short) (ce->red   << (7 + maxval));
    colorCell[i].green = (unsigned short) (ce->green << (7 + maxval));
    colorCell[i].blue =  (unsigned short) (ce->blue  << (7 + maxval));
    colorCell[i].flags = DoRed | DoGreen | DoBlue;
    colorCell[i].pixel = i;
  }


  loadedCmap = 1; // set to 1 so this cmap can not be freed

  int locstatus = XStoreColors(Dpy,privateCmap,colorCell,i);
  if( locstatus == BadColor)
    insertStringtoInfo("bad color \n");
  if( locstatus == BadValue)
    insertStringtoInfo("bad value\n");
  if(status == PRIVATE_E){
    effectiveCmap = privateCmap;
    XSetWindowColormap(Dpy,XtWindow(defaultShell),effectiveCmap);
  }
}

void Color::fitIntoDefaultColormap(){
  colorHist->startLooping();
  int i;
  XColor colr;
  ColorEntry *ce;
  for(i=0;;i++){
    ce = colorHist->nextEntry();
    if(ce == NULL)
      break;
    colr.red =   (unsigned short) (ce->red   << (7 + maxval));
    colr.green = (unsigned short) (ce->green << (7 + maxval));
    colr.blue =  (unsigned short) (ce->blue  << (7 + maxval));
    colr.flags = DoRed | DoGreen | DoBlue;
    ce->pixval = allocateColor(&colr);
  }
}

unsigned long Color::findPixval(unsigned char r,
				unsigned char g, 
				unsigned char b)
{
  if(r == 0 && g == 0 && b == 0)
    return BlackPixel(Dpy,DefaultScreen(Dpy));
  if(r == 0xff && g == 0xff && b == 0xff)
    return WhitePixel(Dpy,DefaultScreen(Dpy));
  
  ColorEntry  *rgb = colorHist->Hfind(r,g,b);
  if(rgb)
    return rgb->pixval;
  else{
    char buffer[100];
    sprintf(buffer,"Can not locate the color (%x %x %x)\n",r,g,b);
    insertStringtoInfo(buffer);
    return 0;
  }
}

unsigned long Color::allocateColor(XColor *colr){ 
  int i, match;
  double rd, gd, bd, dist, mindist;
  unsigned long cindx;
  XColor def_colrs[256];
  int NumCells;
  
  match = XAllocColor(Dpy, effectiveCmap, colr);
  if(match == 0)
    {
      NumCells = DisplayCells(Dpy, DefaultScreen(Dpy));
      for (i=0; i<NumCells; i++)
	{
	  def_colrs[i].pixel = i;
	}
      XQueryColors(Dpy, effectiveCmap, def_colrs, NumCells);
      mindist = 65536.0 * 65536.0;
      cindx = colr->pixel;
      for (i=0; i<NumCells; i++)
	{
	  rd = (def_colrs[i].red - colr->red) / 256.0;
	  gd = (def_colrs[i].green - colr->green) / 256.0;
	  bd = (def_colrs[i].blue - colr->blue) / 256.0;
	  dist = (rd * rd * rd * rd) +
	         (gd * gd * gd * gd) +
	         (bd * bd * bd * bd);
	  if (dist < mindist)
	    {
	      mindist = dist;
	      cindx = def_colrs[i].pixel;
	    }
	}
      colr->pixel = cindx;
      colr->red = def_colrs[cindx].red;
      colr->green = def_colrs[cindx].green;
      colr->blue = def_colrs[cindx].blue;
    }else{
      insertAMayBeFreedCells(colr->pixel); 
    }
  return colr->pixel;
}
void Color::cleanCellsCanBeFreedArray(){
  for(int i=0;i<256;i++)
    CellsCanBeFreed[i] = 0;
}
void Color::insertAMayBeFreedCells(unsigned long x){
  // set it to one so I know it can be freed
  CellsCanBeFreed[(int) x] = 1;
}
void Color::freeCmap(){
  if(loadedCmap == 1){
    insertStringtoInfo("loaded cmap cells can not be freed\n");
    return;
  }
  int i,j,k,resv;
  for(i=0,k=0;i<256;i++){
    if(CellsCanBeFreed[i] == 0)
      continue;
    resv = 0;
    for(j=0;j<NoOfReservedColors;j++){
      if(i == (int) reservedColors[j].pixel){ // reserved color don\'t delete it
	resv = 1;
	break;
      }
    }
    if(resv == 0 && CellsCanBeFreed[i] == 1){
      CellsCanBeFreed[i] = i;
      XFreeColors(Dpy,effectiveCmap,&CellsCanBeFreed[i],1,0);
      k++;
    }
  }
  insertStringtoInfo("Freed <%d> color cells.\n",k);
  allocateAllAvailableCells();
  cleanCellsCanBeFreedArray();
}



