//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "colorHist.h"
#include <libc.h>
#include <string.h>
#include "util.h"

ColorHist::ColorHist(int s)
:Hash(s),size(s),current(0)
{
}
ColorHist::~ColorHist(){
  clearEverything();
}
int   ColorHist::howMany(){
  return count;
}
void ColorHist::makeKey(unsigned char r,unsigned char g, unsigned char b){
  key[0] = r;
  key[1] = g;
  key[2] = b;
  key[3] = '\0';
}
int ColorHist::Hinsert(unsigned char r,unsigned char g, unsigned char b){
  if(count >= size)
    return 0;        // too many colors
  makeKey(r,g,b);
  ENTRY item;
  item.key = key;
  ENTRY *rtn;
  ColorEntry *ce;
  if((rtn = hsearch(item,FIND_E))){
    ((ColorEntry *)rtn->data)->occurrence++;
    return 1;
  } else {
    ENTRY item;
    item.key = new char[strlen(key)+1];
    strcpy(item.key,key);
    ce = new ColorEntry();
    ce->occurrence = ce->pixval = 0;
    ce->red = r;
    ce->green = g;
    ce->blue = b;
    item.data = (char *) ce;
    if(hsearch(item,ENTER_E) == NULL)
      return NULL;
    else
      return 1;
  }
}
ColorEntry *ColorHist::Hfind(unsigned char r,unsigned char g, unsigned char b){
  makeKey(r,g,b);
  ENTRY item;
  item.key = key;
  ENTRY *entry = hsearch(item,FIND_E);
  if(entry)
    return (ColorEntry *) entry->data;
  else
    return NULL;
}
void ColorHist::startLooping(){
  loopIdx = 0;
}
ColorEntry *ColorHist::nextEntry(){
  while(loopIdx < length && table[loopIdx].key == NULL)
    loopIdx++;
  if(loopIdx < length)
    return (ColorEntry *) table[loopIdx++].data;
  else
    return NULL;
}
void ColorHist::clearEverything(){
  ColorEntry *l;
  char *n;
  unsigned int i;	/* Counts table slots */
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      n = table[i].key;
      l = (ColorEntry *) table[i].data;
      if(n)
	delete n;
      if(l)
	delete l;
      table[i].key = NULL;
      table[i].data = NULL;
    }
  }

}
