//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "cutbgImage.h"
#include "util.h"


// Test whether rect1 and rect2 intersect.
static int RectsTouch(int r1x,int r1y,int r1width,int r1height,
	       int r2x,int r2y,int r2width,int r2height)
    {
    if ( r1x + r1width  <= r2x ||
         r1y + r1height <= r2y ||
         r2x + r2width  <= r1x ||
         r2y + r2height <= r1y )
	return False;
    return True;
    }

// Make rect1 the intersection of rect1 and rect2.
static void RectsIntersect(int *r1x,int *r1y,int *r1width,int *r1height,
		    int r2x,int r2y,int r2width,int r2height)
     
{
  if ( ! RectsTouch(*r1x,*r1y,*r1width,*r1height,r2x,r2y,r2width,r2height))
    {
      *r1width = *r1height = 0;
      return;
    }
  if ( r2x + r2width < *r1x + *r1width )
    *r1width = r2x + r2width - *r1x;
  if ( r2y + r2height < *r1y + *r1height )
    *r1height = r2y + r2height - *r1y;
  if ( r2x > *r1x )
    {
      *r1width = *r1x + *r1width - r2x;
      *r1x = r2x;
    }
  if ( r2y > *r1y )
    {
      *r1height = *r1y + *r1height - r2y;
      *r1y = r2y;
    }
}

CutbgImage::CutbgImage(Display *d,Widget w,XImage *bg)
     : Clip(d,w),background(bg)
{
  image = NULL;
}

CutbgImage::~CutbgImage() {
  if(image)
    XDestroyImage(image);
  // don't need to free the following since it is the global background
  // can can not be deleted
  //if(background)
  //  XDestroyImage(background);
  deallocateString();
}
void CutbgImage::dontNeedBackgroungAnyMore(){
  if(background)
    XDestroyImage(background);
  background = NULL;
}
Clip *CutbgImage::dup(){
  CutbgImage *rtn = new CutbgImage(*this);
  rtn->diskFileName  = NULL;
  rtn->shapeFileName = NULL;
  rtn->clipName      = NULL;
  if(background){
    Pixmap pix = XCreatePixmap(Dpy, XtWindow(drawingArea),background->width,
			       background->height,8);
    XPutImage(Dpy,pix,gc,background,0,0,0,0,background->width,background->height);
    rtn->background =  XGetImage(Dpy,pix,0,0,background->width,
				 background->height,0xff,ZPixmap);
    XFreePixmap(Dpy,pix);
  }
  copyto(rtn);
  rtn->RefCounter = 0;
  return rtn;
}
void CutbgImage::MakeImageAndDisplay(OutputWindow *wid,int w, int h, 
				     int current_x,int current_y,int,int){
  MakeImage(w,h,current_x,current_y,0,0);
  if(image && r1width > 0 && r1height > 0)
    XPutImage(wid->Dpy, wid->window, gc, image, 0, 0,
	      r1x,r1y,r1width,r1height);

}
// making an image out of the background image
void CutbgImage::MakeImage(int w, int h, int current_x,int current_y,int,int){
  height =  h;
  width  = w;
  if(w <= 0 || h <= 0){
    image = NULL;
    return;
  }
  if(image)
    XDestroyImage(image);

  // must do the clip here otherwise the XsubImage will crash
  r1x = current_x;
  r1y = current_y;
  r1width = w;
  r1height = h;
  RectsIntersect(&r1x,&r1y,&r1width,&r1height,
		 0,0,background->width,background->height);

  image = XSubImage(background,r1x,r1y,r1width,r1height);

}

