//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "expr.h"
#include <libc.h>
#include <string.h>
#include "built_in.h"
#include "util.h"

Math_Expr::Math_Expr()
:Hstring_to_evaluate(NULL),lineno(0),getc_index(0)
{
  for(int i=0;i<10;i++){
    valuedSymbol[i].name[0] = '\0';
    valuedSymbol[i].value   = 0.0;
  }
}

Math_Expr::Math_Expr(char *s)
:Hstring_to_evaluate(NULL),lineno(0),getc_index(0)
{
  for(int i=0;i<10;i++){
    valuedSymbol[i].name[0] = '\0';
    valuedSymbol[i].value   = 0.0;
  }
  ALLOC_STRCPY(Hstring_to_evaluate,s);
}

Math_Expr::~Math_Expr(){
  delete Hstring_to_evaluate;
}

void Math_Expr::SetInputHstring(char *s){
  lineno = 0;
  getc_index = 0;
  ALLOC_STRCPY(Hstring_to_evaluate,s);
}

char *Math_Expr::getExprInString(){
  return Hstring_to_evaluate;
}

int Math_Expr::yeywrap(){
  return 1;
}

void Math_Expr::yeyerror(char *s)
{
  char buf[200];
  sprintf(buf,"%s <<%s>> \n line = %d pos = %d\n",s,
	 Hstring_to_evaluate,lineno,getc_index);
  insertStringtoInfo(buf);
  parsingStatus = failed_E;
}

int Math_Expr::Getc(){
  if(getc_index > strlen(Hstring_to_evaluate))
    return -1;
  else
    return Hstring_to_evaluate[getc_index++];
}

ParsingStatus Math_Expr::evaluate(){
  lineno = 0;
  getc_index = 0;
  parsingStatus = successful_E;
  yeyparse();
  return parsingStatus;
}

double Math_Expr::GetResult()
{
  return result;
}

double Math_Expr::evalFunc(char *s,double t){
  return evalFunc(s,t,NULL);
}

double Math_Expr::evalFunc(char *s){
  return evalFunc(s,NULL,NULL);
}

double Math_Expr::evalFunc(char *s,double t ,double r){
  double res;
  if(evalBuildInFuncD(s,&res,t,r) == 1){
    return res;
  }
  parsingStatus = failed_E;
  return 0.0;
}

void Math_Expr::setCurrentValue(char *s,double d){
  int i;
  // find whether the symbol exist
  for(i=0;i<10;i++){
    if(strcmp(s,valuedSymbol[i].name) == 0){
      valuedSymbol[i].value = d;
      return;
    }
  }
  // not in the list insert it
  for(i=0;i<10;i++){
    if(strcmp("",valuedSymbol[i].name) == 0){
      strcpy(valuedSymbol[i].name,s);
      valuedSymbol[i].value = d;
      return;
    }
  }
}

double Math_Expr::getCurrentValue(char *s){
  for(int i=0;i<10;i++){
    if(strcmp(s,valuedSymbol[i].name) == 0)
      return valuedSymbol[i].value;
  }
  insertStringtoInfo("Symbol <%s> not defined, assume its value is 0.0\n",s);
  return 0.0;
}

