//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

// This section of code is used to fetch image size.
// Currently, it only knows ppm, gif, and jpeg format.
// There are stubs for other foramts.
// Some code was originated from xv and djpeg and I modified it.
//
//                  Ko-Haw Nieh

#include "config.h"
#include <sys/types.h>
#include <sys/stat.h>
#include "importImage.h"
#include "util.h"

#define  M_SOF0   0xc0
#define  M_SOF1   0xc1
#define  M_SOF9   0xc9
#define  M_SOI    0xd8

ImportImage::ImportImage(char *fn)
     :filename(NULL),imageType(NULL),width(0),height(0),maxval(0),garbage(0),
     fileSize(0)
{
  ALLOC_STRCPY(filename,fn);
  int len = strlen(fn);

  // read the first couple bytes (Magic Number) to determinate the file type
  FILE *fp=fopen(filename,"r");
  if (!fp) {
    imageType = "Can not open file";
    return;
  }
  // get file size
  struct stat sbuf;
  stat(filename,&sbuf);
  fileSize = (int) sbuf.st_size;
  if(fileSize <= 0){
    imageType = "file is empty";
    fclose(fp);
    return;
  }

  unsigned char magicNumber[8];
  fread((char *) magicNumber,8,1,fp);  
  fclose(fp);
  imageType = "UNKNOWN";
  if(strncmp((char *) magicNumber,"GIF87a",6)==0){
    imageType = "GIF87a";
    getGIFinfo();
  }
  if(strncmp((char *) magicNumber,"GIF89a",6)==0){ 
    imageType = "GIF89a";
    getGIFinfo();
  }
  if(strncmp((char *) magicNumber,"VIEW",4)==0 ||
      strncmp((char *) magicNumber,"WEIV",4)==0){
	imageType = "PM";
	getPMinfo();
      }
  if(magicNumber[0] == 'P' && magicNumber[1]>='1' && 
     magicNumber[1]<='6'){
       imageType = "PBM";
       getPBMinfo();
     }
  if(strncmp((char *) magicNumber,"#define",7)==0){
    imageType = "XBM";
    getXBMinfo();
  }

  if(magicNumber[0]==0x59 && (magicNumber[1]&0x7f)==0x26 &&
     magicNumber[2]==0x6a && (magicNumber[3]&0x7f)==0x15){
       imageType = "SUNRASTER";
       getSUNRASTERinfo();
     }
  if(magicNumber[0]==0x52 && magicNumber[1]==0xcc){
    imageType = "UTAHRLE";
    getUTAHRLEinfo();
  }
  if(magicNumber[0]==0xff && magicNumber[1]==0xd8 && 
     magicNumber[2]==0xff){
       imageType = "JFIF";
       getJFIFinfo();
     }
  if((magicNumber[0]=='M' && magicNumber[1]=='M') ||
     (magicNumber[0]=='I' && magicNumber[1]=='I')){
       imageType = "TIFF";
       getTIFFinfo();
     }
  if(strncmp((char *) magicNumber,  "NJPL1I00",8)==0 || /* fixed-len pds */
     strncmp((char *) magicNumber+2,"NJPL1I",  6)==0 || /* vger+other pds */
     strncmp((char *) magicNumber,  "CCSD3ZF", 7)==0 || /* vikng pds browse*/
     strncmp((char *) magicNumber+2,"CCSD3Z",  6)==0 || /* vik. huffman pds*/
     strncmp((char *) magicNumber,  "LBLSIZE=",8)==0){   /* vicar */
       imageType = "PDSVICAR";
       getPDSVICARinfo();
     }
}

ImportImage::~ImportImage() {
  delete filename;
}

void ImportImage::getPBMinfo(){
  FILE  *fp;
  char c, c1;
  garbage = 0;
  fp=fopen(filename,"r");
  if (!fp) {
    imageType = "Can not open file";
    return;
  }
  // read first two bytes 
  //    P1 = ascii bitmap
  //    p2 = ascii greymap
  //    p3 = ascii pixmap     <----------- support
  //    p4 = raw bitmap
  //    p5 = raw grepmap
  //    p6 = raw pixmap     <----------- support
  c = getc(fp);  c1 = getc(fp);
  if(!(c == 'P' && (c1 != '3' || c1 != '6'))){
    insertStringtoInfo("Unsupported image format\n");
    imageType = "Unsupported PPM Format";
    return;
  }
  // read width and height
  width = getint(fp);
  height = getint(fp);
  // read max value 
  maxval = getint(fp);
  if(c1 == '3')
    imageType = "PPM (ASCII)";
  else
    imageType = "PPM (Raw)";
  fclose(fp);
}

int ImportImage::getint(FILE *fp)
{
  int c, i;
  
  /* skip forward to start of next number */
  c = getc(fp);
  while (1) {
    /* eat comments */
    if (c=='#') {   /* if we\'re at a comment, read to end of line */
      while (c != '\n' && c != EOF) c=getc(fp);
    }

    if (c==EOF) return 0;
    if (c>='0' && c<='9') break;   /* we\'ve found what we were looking for */
      
      /* see if we are getting garbage \(non-whitespace\) */
      if (c!=' ' && c!='\t' && c!='\r' && c!='\n' && c!=',') garbage=1;
      
      c = getc(fp);
    }
    
    /* we\'re at the start of a number, continue until we hit a non-number */
  i = 0;
  while (1) {
    i = (i*10) + (c - '0');
    c = getc(fp);
    if (c==EOF) return i;
    if (c<'0' || c>'9') break;
  }

  return i;

}

void   ImportImage::getGIFinfo()
{
  register unsigned char  ch;
  register unsigned char *ptr;

  FILE *fp = fopen(filename,"r");
  if (!fp) {
    imageType = "Can not open file";
    return;
  }

  // read first 256 bytes
  char buf256[256];
  if (fread(buf256, 256, 1, fp) != 1){ 
    imageType = "GIF data read failed";
    fclose(fp);
    return;
  }
  ptr = (unsigned char *) buf256;
  
  ptr += 6;
  
  // Get variables from the GIF screen descriptor
  
  ch = (*ptr++);
  width = ch + 0x100 * (*ptr++);	// screen dimensions
  ch = (*ptr++);
  height = ch + 0x100 * (*ptr++);
  
  ch = (*ptr++);
  int BitsPerPixel = (ch & 7) + 1;
  maxval  = 1 << BitsPerPixel;

  fclose(fp);
}

int ImportImage::JGETC(FILE *fp){
  int c = fgetc(fp);
  return c;
}

long ImportImage::get_2bytes (FILE *fp)
/* Get a 2-byte unsigned integer (e.g., a marker parameter length field) */
{
  long a;
  
  a = JGETC(fp);
  return (a << 8) + JGETC(fp);
}

int ImportImage::next_marker (FILE *fp)
/* Find the next JPEG marker */
/* Note that the output might not be a valid marker code, */
/* but it will never be 0 or FF */
{
  int c, nbytes;

  nbytes = 0;
  do {
    do {			/* skip any non-FF bytes */
      nbytes++;
      c = JGETC(fp);
    } while (c != 0xFF);
    do {			/* skip any duplicate FFs */
      nbytes++;
      c = JGETC(fp);
    } while (c == 0xFF);
  } while (c == 0);		/* repeat if it was a stuffed FF/00 */

  return c;
}

void ImportImage::get_sof (FILE *fp)
/* Process a SOFn marker */
{
  long length;

  length = get_2bytes(fp);
  
  int data_precision = JGETC(fp);
  height   = (unsigned int) get_2bytes(fp);
  width    = (unsigned int) get_2bytes(fp);
  maxval   = 255;
}

void   ImportImage::getJFIFinfo(){
  FILE *fp = fopen(filename,"r");
  if (!fp) {
    imageType = "Can not open file";
    return;
  }
  int c;
  for(;;){
    c = next_marker(fp);
    switch (c) {
    case M_SOF0:
    case M_SOF1:
    case M_SOF9:
      get_sof(fp);
      fclose(fp);
      return;
    default:
      break;
    }
  }
}

void   ImportImage::getPMinfo(){}
void   ImportImage::getXBMinfo(){}
void   ImportImage::getSUNRASTERinfo(){}
void   ImportImage::getUTAHRLEinfo(){}
void   ImportImage::getTIFFinfo(){}
void   ImportImage::getPDSVICARinfo(){}
