//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "locus.h"
#include "util.h"
#include <string.h>
#include <libc.h>

Locus::Locus(int s)
:RefCounter(0),current(0),name(NULL),size(0),incr(1)
{
  allocatedSize = s;
  points = new LocusPoint[s];
}

Locus::Locus(Locus &ls)
:RefCounter(0),current(0),name(NULL),size(0),incr(1)
{
  allocatedSize = size = ls.getSize();
  ALLOC_STRCPY(name,ls.name);
  RefCounter = ls.RefCounter;
  LocusPoint *lp;
  points = new LocusPoint[ls.getSize()];
  for(int i = 0;i < size;i++){
    lp = ls.getPoint(i);
    points[i].x = lp->x;
    points[i].y = lp->y;
  }
}

int Locus::load(char *fileName){
  FILE *fp;
  char *p;
  fp = fopen(fileName,"r");
  if(fp){   
    char line[BUFSIZ+1];
    int i;
    // read number of points in
    if((p = fgets(line, BUFSIZ, fp)) == NULL){
      insertStringtoInfo("Syntax error: in <%s>",fileName);
      return 0;
    }
    size = atoi(line);
    if(points)
      delete points;
    points = new LocusPoint[size];
    if((p = fgets(line, BUFSIZ, fp)) == NULL){
      insertStringtoInfo("syntax error: in <%s>",fileName);
      return 0;
    }
    for(i=0;i<size;i++){
      p = SkipToNextDigit(p);
      if(p == NULL){
        if((p = fgets(line, BUFSIZ, fp)) == NULL){
	  insertStringtoInfo("syntax error: in <%s>",fileName);
	  return 0;
	}
	p = SkipToNextDigit(p);
      }
      points[i].x = atoi(p);
      p = SkipToNextDigit(p);
      points[i].y = atoi(p);
    }
    return 1;
  }else
    return 0;
}

Locus::~Locus(){
  if(points) delete points;
  if(name)   delete name;
}

void Locus::incrRefCounter(){
  RefCounter++;
}

void Locus::decrRefCounter(){
  RefCounter--;
  if(RefCounter < 0)
    RefCounter = 0;
}

int Locus::getRefCounter(){
  return RefCounter;
}

void Locus::depositPoints(int s,LocusPoint *p){
  allocatedSize = -1;  // -1 means allocated size is unknown
  size = s;
  if(points)
    delete points;
  points = p;
}

void       Locus::setName(char *s){
  ALLOC_STRCPY(name,s);
}

char      *Locus::getName(){
  return name;
}

void       Locus::initialize(){
  current = 0;
}

int        Locus::addPoints(int x,int y){
  if(incr > 0 && current > allocatedSize){
    printf("Can not take no more points, ignore all succeeding points\n");
    return -1;
  }
  if(incr < 0 && current < 0){
    printf("Can not take no more points, ignore all succeeding points\n");
    return -1;
  }
  points[current].x = x;
  points[current].y = y;
  current = current + incr;
  return current;
}

void         Locus::endOfAddingPoints(){
  size = current;
}

LocusPoint *Locus::getPoint(int which){
  if(which < 0 || which >= size)
    return NULL;
  else
    return &points[which];
}

int        Locus::getSize(){
  return size;
}

void        Locus::setIncr(int i){
  incr = i;
}

void Locus::writeOut(FILE *fp){
  char buffer[256];
  sprintf(buffer,"%s/%s.locus",EFFECTIVE_DIRECTORY,name);
  fprintf(fp,"locus:%s:%s.locus:%d \n",name,name,size);
  if(writeOut(buffer) == 0)
    insertStringtoInfo("Can not write locus to <%s>\n",buffer);
}

int Locus::writeOut(char *fileName){
  FILE *fp = fopen(fileName,"w+");
  if(fp){
    int i,j;
    fprintf(fp,"%d\n",size);
    for(i=0,j=0;i<size;i++,j++){
      if(j > 0 && j % 10 == 0){
	fprintf(fp,"\n");
      }
      fprintf(fp," (%d,%d) ", points[i].x, points[i].y);
    }
    fprintf(fp,"\n");
    fclose(fp);
    return 1;
  }
  return 0;
}

void Locus::unique(int where){
  LocusPoint *newAry = new LocusPoint[size];
  int i,j;
  int xprev,yprev;
  xprev = newAry[0].x = points[0].x;
  yprev = newAry[0].y = points[0].y;
  for(i=1,j=1;i<where;i++){
    if(xprev == points[i].x && yprev == points[i].y)
      continue;
    else{
      xprev = newAry[j].x = points[i].x; 
      yprev = newAry[j++].y = points[i].y;
    }
  }
  delete points;
  points = newAry;
  size = j;
}
