//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "xet.h"
#include "locusTable.h"
#include <libc.h>
#include <string.h>
#include "util.h"
#include "config.h"

extern Xet   *XET;

LocusTable::LocusTable(int k)
:Hash(k),dup_index(0),size(k)
{
}

LocusTable::~LocusTable(){
}

int LocusTable::Size(){
  return occupy();
}

Locus *LocusTable::dup(char *name){
  Locus *ls = findLocus(name);
  Locus *rtn;
  char buf[100];
  if(ls){
    rtn = new Locus(*ls);
    sprintf(buf,"%s_%d",ls->getName(),dup_index++);
    rtn->setName(buf);
    addLocus(buf,rtn);
  }else
    XET->popupModalInfoBox("It is impossible, locus <%s> not found",name); 
  return rtn;
}

void LocusTable::renameKey(char *Old, char *New){
  ENTRY item;
  item.key = Old;
  ENTRY *rtn;
  if((rtn = hsearch(item,DELETE_E))){
    char *n = rtn->key;
    Locus *l = (Locus *) rtn->data;
    if(n)
      delete n;
    addLocus(New,l);
    delete rtn;
  }else
    printf("It is impossible,Locus <%s> not found\n",Old);
}

// both items are deleted by the deleteLocus member function
int LocusTable::addLocus(char *name,Locus *c){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,FIND_E))){
    XET->popupModalInfoBox("locus <%s> already exist or table is full\n",name);
    return -1;
  } else {
    ENTRY item;
    item.key = new char[strlen(name)+1];
    strcpy(item.key,name);
    item.data = (char *) c;
    hsearch(item,ENTER_E);
    char tmpbuf[256];
    sprintf(tmpbuf,LOCUS_FORMAT,name,c->getRefCounter(), c->getSize());
    XET->locusInfo(tmpbuf);
    return 1;
  }
}

void LocusTable::deleteLocus(){
  clearEverything();
}

void LocusTable::deleteLocus(char *name){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,DELETE_E))){
    char *n = rtn->key;
    Locus *l = (Locus *) rtn->data;
    if(n)
      delete n;
    if(l)
      delete l;
    delete rtn;
  }
}

Locus *LocusTable::findLocus(char *key){
  ENTRY item;
  item.key = key;
  ENTRY *entry = hsearch(item,FIND_E);
  if(entry){
    return (Locus *) entry->data;
  }
  else
    return NULL;
}

void LocusTable::buildLocusXmList(Widget w){
  char buf[100];
  int i,j;	
  char *c;
  char **tmp = new char *[length];
  for(i = j = 0; i < length; i++){
    if(table[i].key != NULL){
      sprintf(buf,LOCUS_FORMAT,table[i].key,
	      ((Locus *)(table[i].data))->getRefCounter(),
	      ((Locus *)(table[i].data))->getSize());
      c = new char[strlen(buf)+1];
      strcpy(c,buf);
      tmp[j++] = c;
    }
  }
  SortString(tmp,j);
  deleteAllItemsOfXmList(w);
  for(i=0;i<j;i++){
    PutTextInListWidget(w,tmp[i]);
    delete tmp[i];
  }
  delete tmp;
}

void LocusTable::writeOut(FILE *fp){
  unsigned int i;	/* Counts table slots */
  fprintf(fp,"## list of locus\n");
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      fprintf(fp,"##Locus Name : %s\n",table[i].key);
      fprintf(fp," \n");
      ((Locus *)(table[i].data))->writeOut(fp);
    }
  }
  fprintf(fp,"\n## end of locus listing \n");
}

void   LocusTable::clearEverything(){
  Locus *l;
  char *n;
  unsigned int i;	/* Counts table slots */
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      n = table[i].key;
      l = (Locus *) table[i].data;
      if(n)
	delete n;
      if(l)
	delete l;
      table[i].key = NULL;
      table[i].data = NULL;
    }
  }
  count = 0;
  XET->locusInfo(NULL);
}
