//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include <sys/types.h>
#include <sys/stat.h>
#include "ppmImage.h"
#include "util.h"
#include "color.h"
#include <math.h>

extern Color *currentCmap;

PPMImage::PPMImage(Display *d,Widget w,char *fn)
: Clip(d,w),destroyFlag(0),filename(0)
{
  ALLOC_STRCPY(filename,fn);
}

PPMImage::~PPMImage() {
  if(destroyFlag == 0 && image)
    XDestroyImage(image);
  deallocateString();
  if(filename)
    delete filename;
}

Clip *PPMImage::dup(){
  PPMImage *rtn = new PPMImage(*this);
  rtn->diskFileName  = NULL;
  rtn->shapeFileName = NULL;
  rtn->clipName      = NULL;
  rtn->filename      = NULL;
  ALLOC_STRCPY(rtn->filename,filename);
  copyto(rtn);
  rtn->RefCounter = 0;
  return rtn;
}

void PPMImage::setDestroyFlag(int d){
  destroyFlag = d;
}

int PPMImage::loadPPM(){
  char buf[100];
  FILE  *fp;
  char c, c1;
  garbage = 0;

  fp=fopen(filename,"r");
  if (!fp) {
    insertStringtoInfo("Can not open image file <%s>\n",filename);
    return -1;
  }
  struct stat sbuf;
  stat(filename,&sbuf);
  int fileSize = (int) sbuf.st_size;
  if(fileSize <= 0){
    insertStringtoInfo("image file <%s> has zero length \n",filename);
    fclose(fp);
    return -1;
  }

  // read first two bytes 
  //    P1 = ascii bitmap
  //    p2 = ascii greymap
  //    p3 = ascii pixmap     <----------- support
  //    p4 = raw bitmap
  //    p5 = raw grepmap
  //    p6 = raw pixmap     <----------- support
  c = getc(fp);  c1 = getc(fp);
  if(!(c == 'P' && (c1 != '3' || c1 != '6'))){
    char khn[3];
    khn[0] = c;
    khn[1] = c1;
    khn[2] = '\0';
    insertStringtoInfo("Unsupported ppm image format <%s>\n",khn);
    return -1;
  }
  // read width and height
  width = getint(fp);
  height = getint(fp);
  // read max value 
  int maxval = getint(fp);
  
  if(maxval > 255){
    insertStringtoInfo("It is impossible ppm can not have maxval > 255 \n");
    return -1;
  }
  currentCmap->setMaxval(maxval);

  pic = new unsigned char [width*height*3];
  unsigned char *pix;  int i,j;
  if(c1 == '3'){ 
    for (i=0, pix=pic; i<height; i++) {
      for (j=0; j<width*3; j++, pix++)
	*pix = getint(fp);
    }
  }else{
    fread((char *)pic, 1, width*height*3, fp);
  }
  fclose(fp);
  if (garbage == 1){
    insertStringtoInfo("Garbage in the ppm image file <%s>\n",filename);
    return -1;
  }
  unsigned char  r,g,b;

  for (i=0, pix=pic; i<height; i++) {
    for (j=0; j<width; j++){
      r = *pix++;
      g = *pix++;
      b = *pix++;
      if(currentCmap->registerNewColor(r,g,b) == 0){
	insertStringtoInfo("too many colors, Xet can not deal with it, use ppmquant to reduce the number of colors\n");
	return -1;
      }
    }
  }
  sprintf(buf,"image: %s : width = %d height = %d\n",filename,width, height);
  insertStringtoInfo(buf);
  return 1;
}

void PPMImage::buildXimage(){
  insertStringtoInfo("Building XImage -- %s\n",filename);
  unsigned char *pix;  int i,j;
  unsigned char  r,g,b;
  depth =  DefaultDepth(Dpy, DefaultScreen(Dpy));
  linepad = 8 - (width % 8);
  linepad = 0;
  bit_data = (unsigned char *)new char[((width+linepad) * height) + 1];
  bytesperline = (width + linepad) * depth / 8;
  unsigned char *pi = bit_data;
  for (i=0, pix=pic; i<height; i++) {
    for (j=0; j<width; j++){
      r = *pix++;
      g = *pix++;
      b = *pix++;
      *pi++  = (unsigned char) currentCmap->findPixval(r,g,b);
    }
    for(int kk=0;kk<linepad;kk++)
      *pi++ = 0;
  }
  
  image = XCreateImage(Dpy, DefaultVisual(Dpy, DefaultScreen(Dpy)), depth,
		       ZPixmap, 0, (char *)bit_data,
		       (width + linepad), height, 8, bytesperline); 
  delete pic;
  
  
}

int PPMImage::getint(FILE *fp)
{
  int c, i;

  /* skip forward to start of next number */
  c = getc(fp);
  while (1) {
    /* eat comments */
    if (c=='#') {   /* if we're at a comment, read to end of line */
      while (c != '\n' && c != EOF) c=getc(fp);
    }

    if (c==EOF) return 0;
    if (c>='0' && c<='9') break;   /* we've found what we were looking for */

    /* see if we are getting garbage (non-whitespace) */
    if (c!=' ' && c!='\t' && c!='\r' && c!='\n' && c!=',') garbage=1;

    c = getc(fp);
  }


  /* we're at the start of a number, continue until we hit a non-number */
  i = 0;
  while (1) {
    i = (i*10) + (c - '0');
    c = getc(fp);
    if (c==EOF) return i;
    if (c<'0' || c>'9') break;
  }

  return i;

}
