//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "xet.h"
#include "procSpooler.h"
#include <stdio.h>
#include <osfcn.h>
#include <signal.h>
#include "util.h"
#include "wrapper.h"

extern Xet        *XET;

extern void sig_child();

#define	tst(a,b)	(currentProcEntry->mode[0] == 'r'? (b) : (a))
#define	RDR	0
#define	WTR	1

ProcSpooler::ProcSpooler()
:sizeOfQueue(0),procEntryQueue(NULL),currentProcEntry(NULL),
spoolerStatus(RESUME_E)
{
}

ProcSpooler::~ProcSpooler(){
}

SpoolerStatus ProcSpooler::getSpoolerStatus(){
  return spoolerStatus;
}

void ProcSpooler::setSpoolerStatus(SpoolerStatus s){
  spoolerStatus = s;
  if(spoolerStatus == RESUME_E)
    if(!currentProcEntry && procEntryQueue)
      startNextOne();
}

void ProcSpooler::finishUpCurrent(int ){
  // finish up the current one and start the next one
  if(currentProcEntry){
    XET->removeXtAddInput(currentProcEntry->xfd);
    close(currentProcEntry->fd);
    delete currentProcEntry;
    currentProcEntry = NULL;
    startNextOne();
  }else
    XET->popupModalInfoBox("impossible error occured in proc spooler\n");
}

void ProcSpooler::startNextOne(){
  if(procEntryQueue && spoolerStatus == RESUME_E){
    currentProcEntry = procEntryQueue;
    procEntryQueue = procEntryQueue->nextEntry;
    sizeOfQueue--;
    execCurrent();
  }else
    XET->updateSpoolerPanel();
}

void ProcSpooler::moveDown(int slot){
  move(slot,-1);
}

void ProcSpooler::moveUp(int slot){
  move(slot,1);
}

void ProcSpooler::move(int slot, int dir){
  if(slot <= 1){
    XET->popupModalInfoBox("can not move this one, it is already scheduled\n");
    return;
  }
  ProcEntry *del,*tmp;
  if(slot == 2 && procEntryQueue){
    if(dir == 1){
      XET->popupModalInfoBox("can not move this one, it is already up in front\n");
      return;
    }
    if(dir == -1){
      del = procEntryQueue;
      procEntryQueue = procEntryQueue->nextEntry;
      if(procEntryQueue){
	tmp = procEntryQueue;
	while(tmp->nextEntry){
	  tmp = tmp->nextEntry;
	}
	tmp->nextEntry = del;
	del->nextEntry = NULL;
	XET->updateSpoolerPanel();
	return;
      }else{  // only one in the queue
	procEntryQueue = del;
	return;
      }
    }
  }
  int idx = 3;
  if(procEntryQueue){
    tmp = procEntryQueue;
    while(tmp){
      if(idx == slot && tmp->nextEntry){
	del = tmp->nextEntry;
	tmp->nextEntry = del->nextEntry;
	break;
      }
      tmp = tmp->nextEntry;
      idx++;
    }
    if(dir == 1){  // move up
      del->nextEntry = procEntryQueue;
      procEntryQueue = del;
    }
    if(dir == -1){ // move down
      tmp = procEntryQueue;
      while(tmp->nextEntry){
	tmp = tmp->nextEntry;
      }
      tmp->nextEntry = del;
      del->nextEntry = NULL;
    }
  }
  XET->updateSpoolerPanel();
}

ProcEntry *ProcSpooler::getCurrentEntry(){
  return currentProcEntry;
}

void ProcSpooler::registerAProcEntry(char *cmd,char *mode, int f){
  registerAProcEntry(cmd,mode,"","",f);
}

void ProcSpooler::registerAProcEntry(char *cmd,char *mode,
				    char *fileName, char *clipName, int f)
{
  ProcEntry *pe = new ProcEntry();
  ALLOC_STRCPY(pe->command,cmd);
  ALLOC_STRCPY(pe->clipName,clipName);
  ALLOC_STRCPY(pe->fileName,fileName);
  ALLOC_STRCPY(pe->mode,mode);
  pe->flag = f;
  if(currentProcEntry == NULL){ // no subprocess is running
    currentProcEntry = pe;
    execCurrent();
  }else{  // add to the queue and increment sizeOfQueue
    ProcEntry *tmp = procEntryQueue;
    if(tmp == NULL){
      procEntryQueue = pe;
      sizeOfQueue++;
      XET->updateSpoolerPanel();
      return;
    }
    while(tmp->nextEntry){
      tmp = tmp->nextEntry;
    }
    tmp->nextEntry = pe;
    XET->updateSpoolerPanel();
  }
}

void ProcSpooler::execCurrent(){
  if(currentProcEntry == NULL){
    return;
  }
  int p[2];
  register myside, hisside, pid;
  
  if(pipe(p) < 0){
    XET->popupModalInfoBox("can not create pipe \n");
    return ;
  }
  myside = tst(p[WTR], p[RDR]);
  hisside = tst(p[RDR], p[WTR]);

  signal(SIGCLD, (SIG_PF) sig_child);  
  if((pid = fork()) == 0) {
    /* myside and hisside reverse roles in child */
    close(myside);
    if (hisside != tst(0, 1)) {
      dup2(hisside, tst(0, 1));
      close(hisside);
    }
    execl("/bin/sh", "sh",  "-c", currentProcEntry->command, 0);
    _exit(1);
  }

  if(pid == -1) {
    close(myside);
    close(hisside);
    XET->popupModalInfoBox("can not spawn subprocess\n");
    return;
  }
  close(hisside);

  currentProcEntry->pid = pid;
  currentProcEntry->fd  = myside;
  currentProcEntry->xfd = XET->registerXtAddInput(currentProcEntry->fd,
						 (XtInputCallbackProc)ReadForkResult,
						 NULL);
  insertStringtoInfo("Spawn a subprocess for image transformation\n");
  insertStringtoInfo("%s\n",currentProcEntry->command);
  // now update the spooler panel
  XET->updateSpoolerPanel();
}

void ProcSpooler::Delete(int slot){
  if(slot <= 1){
    XET->popupModalInfoBox("can not delete this one, it is already scheduled\n");
    return;
  }
  ProcEntry *del;
  if(slot == 2 && procEntryQueue){
    del = procEntryQueue;
    procEntryQueue = procEntryQueue->nextEntry;
    delete del;
    sizeOfQueue--;
    XET->updateSpoolerPanel();
    return;
  }
  int idx = 3;
  if(procEntryQueue){
    ProcEntry *tmp = procEntryQueue;
    while(tmp){
      if(idx == slot && tmp->nextEntry){
	del = tmp->nextEntry;
	tmp->nextEntry = del->nextEntry;
	delete del;
	sizeOfQueue--;
	break;
      }
      tmp = tmp->nextEntry;
      idx++;
    }
  }
  XET->updateSpoolerPanel();
}

void       ProcSpooler::DeleteAll(){
  if(currentProcEntry){
    XET->removeXtAddInput(currentProcEntry->xfd);
    kill(currentProcEntry->pid, -9);
    delete currentProcEntry;
    currentProcEntry = NULL;
  }
  ProcEntry *tmp = procEntryQueue;
  ProcEntry *del;
  while(tmp){
    del = tmp;
    tmp = tmp->nextEntry;
    delete del;
  }
  procEntryQueue = NULL;
  XET->updateSpoolerPanel();
}

int ProcSpooler::Howmany(){
  int rtn = 0;
  if(currentProcEntry){
    rtn++;
  }
  ProcEntry *tmp = procEntryQueue;
  while(tmp){
    rtn++;
    tmp = tmp->nextEntry;
  }
  return rtn;
}
int ProcSpooler::haveYouSeenThisName(char *name){
  if(currentProcEntry){
    if(strcmp(currentProcEntry->clipName,name) == 0)
      return 1;
  }
  ProcEntry *tmp = procEntryQueue;
  while(tmp){
    if(strcmp(tmp->clipName,name) == 0)
      return 1;
    tmp = tmp->nextEntry;
  }
  return 0;
}

void ProcSpooler::buildQueueXmList(Widget list){
  char buffer[256];
  deleteAllItemsOfXmList(list);
  if(currentProcEntry){
    sprintf(buffer,SPOOLER_FORMAT,currentProcEntry->clipName,currentProcEntry->command);
    PutTextInListWidget(list,buffer);
  }else{
    if(procEntryQueue)
      PutTextInListWidget(list,"********  spooler is suspended\n");
    else
      PutTextInListWidget(list,"********  queue is empty\n");
  }
  ProcEntry *tmp = procEntryQueue;
  while(tmp){
    sprintf(buffer,SPOOLER_FORMAT,tmp->clipName,tmp->command);
    PutTextInListWidget(list,buffer);
    tmp = tmp->nextEntry;
  }
}
