//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#ifndef RUN_BASE_H
#define RUN_BASE_H

#include <X11/Intrinsic.h>
#include "clip.h"
#include "expr.h"
#include "locus.h"
#include "clipList.h"
#include "outputWindow.h"

class Xet;
class Scheduler;

enum TrailingMode { Preserve_E, Wipeout_E, Leave_E, Rainbow_E, };
enum RunMode      { Fixed_E, Glide_E, Random_E, Fly_E, Blink_E, Locus_E, Movie_E, };
enum BoundaryMode { Wrap_E, BounceBack_E, Disappear_E, };
enum TermCriteria { NOT_APPLICATBLE_E, MAX_MOVES_E, POSITION_E, WALL_CLOCK_E, 
		    TIME_INTERVAL_E, };

class RunBase {
friend class Xet;
friend class RunMovie;
friend class Scheduler;
protected:
  char          *name;
  unsigned long  interval;
  XtIntervalId   xid;
  RunBase       *next;
  RunBase       *prev;
  int            status;       // when writing out an xetfile it takes a while
                               // therefore I must suspend all running objects
                               // after finishing writing I will resume them
                               // without doing so will cause X protocol error

  Clip          *clip;
  Clip          *x_trailing_image;
  Clip          *y_trailing_image;
  ClipList      *clipList;
  Display       *Dpy;
  GC             gc;
  RunMode        runMode;
  TermCriteria   terminationCriteria;
  TrailingMode   trailingMode;
  OutputWindow  *drawingArea;
  XtIntervalId   intervalXid;
  int            clip_height;
  int            clip_width;
  int            current_x;
  int            current_y;
  int            d_x; 
  int            d_y;
  int            intervalIndicator;
  int            nextMoveCounter;
  int            tc_max_interval;
  int            tc_max_moves;
  int            tc_x,tc_y,tc_width,tc_height;
  static void    work(XtPointer xptr, XtIntervalId *);
public:
  RunBase(Display *d,OutputWindow  *da,ClipList *clpList,RunMode rm);
  virtual ~RunBase();

  virtual BoundaryMode getBoundaryMode();
  virtual RunBase     *Copy();
  virtual int          nextMove();
  virtual int          setFlyEquation(char *Field1,char *Field2,char *Field3);
  virtual int          setFlyEquationI(char *,char *,int,int,int,int);
  virtual void	       reset();
  virtual void	       reverse();
  virtual void	       setBoundaryMode(BoundaryMode);
  virtual void	       setBoundingBox(int,int,int,int);
  virtual void	       setLocus(Locus *);
  virtual void	       setTranslation(int x, int y);
  virtual void	       start(int x, int y, int dx, int dy);
  virtual void	       writeToDisk(FILE *fp);
  virtual void         setOutputWindow(OutputWindow *);
  Clip		      *getClip();
  ClipList	      *getclipList()  { return clipList; }
  OutputWindow        *getOutputWindow();
  RunMode	       getRunMode();
  TermCriteria	       getTermCriteria();
  TrailingMode	       getTrailingMode();
  char		      *getTermCondInString();
  char		      *getTermCondInString(TermCriteria);
  char		      *getTermCondValueInString();
  char		      *getTermCondValueInString(TermCriteria);
  char		      *trailingModeInString(TrailingMode);
  char		      *boundaryModeInString(BoundaryMode);
  char		      *getRunModeInString();
  char		      *getName();
  int		       shouldTerminate();
  void		       reverseClip();

  void		       setTermCriteria(TermCriteria, char *);
  void		       setTermCriteria(char *, char *);
  void		       resetTerminationCond();
  void		       writeOutTermCond(FILE *fp);
  void		       setNextClip();
  void		       setclipList(ClipList *);
  void		       setName(char *s);
  void		       incrRefcounter();
  void		       decrRefCounter();
  void		       setTrailingMode(TrailingMode opt = Wipeout_E);
  void		       jump(int DisplayStatus);
  void		       placeToNewLocation(int rx, int ry);
  void		       takeCareOfTrailingImage();
  void		       takeCareOfOldLocation(int rx, int ry);
  void		       isCollide(RunBase *c);

};

#endif

