//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "run_fixed.h"
#include "util.h"
#include <libc.h>
#include "config.h"


RunFixed::RunFixed(Display *d,OutputWindow *w,ClipList *im)
     :RunBase(d,w,im,Fixed_E),dspFlag(0)
{
}

RunBase *RunFixed::Copy(){
  RunFixed *rf =  new RunFixed(*this);
  rf->clipList = new ClipList(*clipList);
  STRCPY(rf->name,name);
  return rf;
}

void RunFixed::setDspFlag(int i){
  dspFlag = i;
}

RunFixed::~RunFixed(){
  if(intervalXid != 0){
    XtRemoveTimeOut(intervalXid);
    intervalXid = 0;
  }
  if(clipList) delete clipList;
  if(name) delete name;
}

void RunFixed::writeToDisk(FILE *fp){
  if(fp){
    fprintf(fp,"fixed:");
    clipList->writeOut(fp);
    fprintf(fp,":%s:%d:%d:%d",
	    trailingModeInString(trailingMode),
	    start_x,start_y,
	    interval);
    writeOutTermCond(fp);
    fprintf(fp,"\n");
  }
}

void RunFixed::start(int x, int y, int dx, int dy){
  start_x = current_x = x;
  start_y = current_y = y;
  start_dx = d_x = dx;
  start_dy = d_y = dy;
  clipList->reset();
}

void RunFixed::reset(){
  resetTerminationCond();
  current_x = start_x;
  current_y = start_y;
  d_x = start_dx;
  d_y = start_dy;
  clipList->reset();
}

int RunFixed::nextMove(){
  setNextClip();
  d_x = d_y = 0;
  if(dspFlag != 1)
    jump(1);
  if(shouldTerminate() == 1){
    return 1;
  }
  if(dspFlag == 1){
    XClearWindow(drawingArea->Dpy,drawingArea->window);
    clip->DisplayImage(drawingArea);
    insertStringtoInfo("Display clip <%s> \n",clip->getClipName());
  }
  else
    clip->DisplayImage(drawingArea, current_x, current_y);
  return 0;
}
