//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "run_glide.h"
#include "util.h"
#include <libc.h>
#include "config.h"


RunGlide::RunGlide(Display *d,OutputWindow *w,ClipList *im)
:RunBase(d,w,im,Glide_E),bounding_x(0),bounding_y(0),
 bounding_width(100),bounding_height(100)
{
}

RunGlide::~RunGlide(){
  if(intervalXid != 0){
    XtRemoveTimeOut(intervalXid);
    intervalXid = 0;
  }
  if(clipList) delete clipList;
  if(name) delete name;
}

RunBase *RunGlide::Copy(){
  RunGlide *rb =  new RunGlide(*this);
  rb->clipList = new ClipList(*clipList);
  STRCPY(rb->name,name);
  return rb;
}

void RunGlide::writeToDisk(FILE *fp){
  if(fp){
    fprintf(fp,"glide:");
    clipList->writeOut(fp);
    fprintf(fp,":%s:%d:%d:%d:%d:%d:%d:%d:%d:%d",
	    trailingModeInString(trailingMode),
	    start_x,start_y,
	    start_dx,start_dy,
	    bounding_x,bounding_y,bounding_width,bounding_height,
	    interval);
    writeOutTermCond(fp);
    fprintf(fp,"\n");
  }
}
void RunGlide::setBoundingBox(int x,int y ,int w,int h){
  bounding_x = (x <= 0) ? 0 : x;
  bounding_y = (y <= 0) ? 0 : y;
  
  bounding_width = (w <= 0 || (x+w) >= (int)drawingArea->da_width) ? 
                     (int)drawingArea->da_width - x : w;
  bounding_height = (h <= 0 || (y+h) >= (int)drawingArea->da_height) ? 
                     (int)drawingArea->da_height - y: h;
  adjustInitialXY();
}
void RunGlide::adjustInitialXY(){
  if(current_x < bounding_x || current_x >(bounding_x + bounding_width))
    current_x = bounding_x + bounding_width / 2;
  if(current_y < bounding_y || current_y > (bounding_y + bounding_height))
    current_y = bounding_y + bounding_height / 2;
}
void RunGlide::start(int x, int y, int dx, int dy){
  start_x = current_x = x;
  start_y = current_y = y;
  start_dx = d_x = dx;
  start_dy = d_y = dy;
  adjustInitialXY();
  clipList->reset();
}

void RunGlide::reset(){
  resetTerminationCond();
  current_x = start_x;
  current_y = start_y;
  d_x = start_dx;
  d_y = start_dy;
  clipList->reset();
  adjustInitialXY();
}

void RunGlide::reverse(){
  d_x = - d_x;
  d_y = - d_y;
}

int  RunGlide::nextMove(){
  if(shouldTerminate() == 1){
    takeCareOfOldLocation(current_x,current_y);
    return 1;
  }
  setNextClip();
  if(abs(d_x) >= clip_width || abs(d_y) >= clip_height){
    jump(1);
    if(shouldTerminate() == 1){
      return 1;
    }
    clip->DisplayImage(drawingArea, current_x, current_y);
    return 0;
  }
  if(shouldTerminate() == 1){
    takeCareOfOldLocation(current_x,current_y);
    return 1;
  }  // calculate the next position
  current_x += d_x;
  current_y += d_y;
  takeCareOfTrailingImage();
  clip->DisplayImage(drawingArea, current_x, current_y);
  
  // determinate the next direction
  if((current_x + clip_width)  > (bounding_width + bounding_x))  
    d_x = - d_x;
  if((current_y + clip_height) > (bounding_height+ bounding_y))
    d_y = - d_y;
  if(current_x < bounding_x)  d_x = - d_x;
  if(current_y < bounding_y)  d_y = - d_y;
  return 0;
}



