//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "xet.h"
#include "scheduler.h"
#include <libc.h>
#include "util.h"
#include "run_movie.h"

extern Xet       *XET;

Scheduler::Scheduler(XtAppContext a, int s)
:Hash(s),app(a),dup_index(1),noOfObjects(0),activeMovie(NULL)
{
}
Scheduler::~Scheduler(){
}
void Scheduler::renameKey(char *Old, char *New){
  ENTRY item;
  item.key = Old;
  ENTRY *rtn;
  if((rtn = hsearch(item,DELETE_E))){
    char *n = rtn->key;
    RunBase *l = (RunBase *) rtn->data;
    if(n)
      delete n;
    addRunObject(New,l,(l->xid)?1:2);
    delete rtn;
  }else
    insertStringtoInfo("It is impossible,Run Object <%s> not found\n",Old);
}

RunBase *Scheduler::findRunObject(char *key){
  if(key){
    ENTRY item;
    item.key = key;
    ENTRY *entry = hsearch(item,FIND_E);
    if(entry){
      return (RunBase *) entry->data;
    }
    else
      return NULL;
  }else
    return NULL;
}

RunBase *Scheduler::dup(char *key){
  RunBase *ro = findRunObject(key);
  if(ro){
    char buff[200];
    RunBase *ronew = ro->Copy();
    sprintf(buff,"%s_%d",key,dup_index++);
    ALLOC_STRCPY(ronew->name,buff);
    if(ro->xid) // the originator is running so schedule it
      addRunObject(ronew->name,ronew,1);
    else  // the original is freeze, so don\'t schedule it
      addRunObject(ronew->name,ronew,2);
    return ronew;
  }else{
    XET->popupModalInfoBox("can not dup run object, <%s> not found",key);
    return NULL;
  }
}
void Scheduler::addRunObject(char *name, RunBase *c, int flag){
  addRunObject(name,c,c->interval,flag);
}
void Scheduler::addRunObject(char *name, RunBase *c,
			    unsigned long interval, int flag){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,FIND_E))){
    insertStringtoInfo("RunBase <%s> already exist\n",name);
    return;
  } else {
    ENTRY item;
    item.key = new char[strlen(name)+1];
    strcpy(item.key,name);
    c->name = new char[strlen(name)+1];
    strcpy(c->name,name);
    c->interval = interval;
    item.data = (char *) c;
    hsearch(item,ENTER_E);
    if(flag == 1)
      c->xid =  XtAppAddTimeOut(app,interval,
				 (XtTimerCallbackProc) Scheduler::work, 
				 c);
    // now increment refcounter on each clip
    c->incrRefcounter();
    c->reset();
  }
  noOfObjects++;
  XET->updateXetStatistic();
  char tmpbuf[256];
  sprintf(tmpbuf,SCHEDULER_FORMAT,
	  c->name,
	  c->getRunModeInString(),
	  c->interval,
	  (c->getclipList() == 0) ? 0 : c->getclipList()->Size(),
	  (c->xid == 0) ? "Frozen " : "Running"); 
  XET->buildSchedulerList(tmpbuf);

}
int Scheduler::writeOut(char *fileName,char *ro){
  FILE *fp = fopen(fileName,"w+");
  if(fp){
    RunBase *cat = findRunObject(ro);
    if(cat){
      fprintf(fp,"scheduler:%s:%s:",ro,
	      (cat->getOutputWindow()->da_area) ? "" : "root");
      cat->writeToDisk(fp);
    }else
      insertStringtoInfo("It is impossible,Run Object <%s> not found\n",ro);
    fclose(fp);
    return 1;
  }
  return 0;
}
void Scheduler::writeOut(FILE *fp){
  unsigned int i;	/* Counts table slots */
  fprintf(fp,"## list of clips\n");
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      fprintf(fp,"scheduler:%s:%s:",table[i].key,
	      (((RunBase *)(table[i].data))->getOutputWindow()->da_area) ? "" : "root");   
      ((RunBase *)(table[i].data))->writeToDisk(fp);
      fprintf(fp,"#\n");
      fprintf(fp,"# ----------------------------------\n");
      fprintf(fp,"#\n");
    }
  }
}
void Scheduler::freeze(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro && ro->xid != 0){
	XtRemoveTimeOut(ro->xid);
	ro->xid = 0;
      }
    }
  }
  XET->updateXetStatistic();
}
void Scheduler::unfreeze(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro && ro->xid == 0){
	ro->xid = XtAppAddTimeOut(app,ro->interval,
				  (XtTimerCallbackProc) Scheduler::work, 
				  ro);
      }
    }
  }
  XET->updateXetStatistic();
}
int  Scheduler::isFrozen(char *key){
  RunBase *ro = findRunObject(key);
  if(ro && ro->xid == 0)
    return 1;
  else
    return 0;
}
void Scheduler::setOutputWindow(char *key,OutputWindow *ow){
  RunBase *ro = findRunObject(key);
  if(ro){
    ro->setOutputWindow(ow);
  }
}
void Scheduler::setOutputWindow(OutputWindow *ow){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro){
	ro->setOutputWindow(ow);
      }
    }
  }
}

void Scheduler::reverse(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro){
	ro->reverse();
      }
    }
  }
}
void Scheduler::reverse(char *key){
  RunBase *ro = findRunObject(key);
  if(ro){
    ro->reverse();
  }
}
void Scheduler::reverseClip(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro){
	ro->reverseClip();
      }
    }
  }
}
void Scheduler::reverseClip(char *key){
  RunBase *ro = findRunObject(key);
  if(ro){
    ro->reverseClip();
  }
}

void Scheduler::reset(char *key){
  RunBase *ro = findRunObject(key);
  if(ro){
    ro->reset();
  }
}
void Scheduler::reset(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro){
	ro->reset();
      }
    }
  }
}
void Scheduler::suspend(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro && ro->xid != 0){
	XtRemoveTimeOut(ro->xid);
	ro->xid = 0;
	ro->status = 1;
      }
    }
  }
  XET->flush();
}
void Scheduler::resume(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro && ro->status == 1 && ro->xid == 0){
	ro->xid = XtAppAddTimeOut(app,ro->interval,
				  (XtTimerCallbackProc) Scheduler::work, 
				  ro);
	ro->status = 0;
      }
    }
  }
  XET->flush();
}
void Scheduler::freeze(char *key){
  RunBase *ro = findRunObject(key);
  if(ro && ro->xid != 0){
    XtRemoveTimeOut(ro->xid);
    ro->xid = 0;
  }
  XET->updateXetStatistic();
}
// the key may be contains \',\' which  means it is a list of keys
// it is used by the command line switch \'-s\'
void Scheduler::unfreeze(char *keys){
  char *key = keys;
  int thatsit = 0;
  for(;;){
    // find the  \',\'
    while(*keys != ',' && *keys != '\0')
      keys++;
    if(*keys == '\0')
      thatsit = 1;
    *keys++ = '\0';
    RunBase *ro = findRunObject(key);
    if(ro && ro->xid == 0){
      ro->xid = XtAppAddTimeOut(app,ro->interval,
				(XtTimerCallbackProc) Scheduler::work, 
				ro);
    }
    key = keys;
    if(thatsit == 1)
      break;
  }
  XET->updateXetStatistic();
}
void Scheduler::schedulerDeleteAll(){
  clearEverything();
}
void Scheduler::clearEverything(){
  unsigned int i;	/* Counts table slots */
  RunBase *ro;
  for(i = 0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro && ro->xid != 0)
	XtRemoveTimeOut(ro->xid);
      delete table[i].key;
      ro->decrRefCounter();
      delete ro; 
      table[i].key = NULL;
      table[i].data = NULL;
    }
  }
  noOfObjects = 0;
  count = 0;
  MovieDeleteAll();
  XET->updateXetStatistic();
  XET->buildSchedulerList(NULL);
}

void Scheduler::Delete(char *name){
  ENTRY item;
  item.key = name;
  ENTRY *rtn;
  if((rtn = hsearch(item,DELETE_E))){
    char *n = rtn->key;
    RunBase *ro = (RunBase *) rtn->data;
    if(ro && ro->xid != 0)
      XtRemoveTimeOut(ro->xid);
    ro->decrRefCounter();
    delete n;
    delete ro;
    delete rtn;
    noOfObjects--;
    XET->updateXetStatistic();
  }
}
int  Scheduler::howmanyRunning(){
  int i,rtn = 0;
  RunBase *ro;
  for(i=0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      if(ro && ro->xid != 0) 
	rtn++;
    }
  }
  return rtn;
}
void Scheduler::buildSchedulerXmList(Widget w){
  char buf[100];
  int i,j;	
  char *c;
  RunBase *ro;
  char **tmp = new char *[length];
  for(i=j=0; i < length; i++){
    if(table[i].key != NULL){
      ro = (RunBase *)(table[i].data);
      sprintf(buf,SCHEDULER_FORMAT,
	      (char *) table[i].key,
	      ro->getRunModeInString(),
	      ro->interval,
	      (ro->getRunMode() == Movie_E) 
	      ? 
	      ((RunMovie *)ro)->howmanyObjects()
	      :
	      ((ro->getclipList() == 0) ? 0 : ro->getclipList()->Size()),
	      (ro->xid == 0) ? "Frozen " : "Running");
      c = new char[strlen(buf)+1];
      strcpy(c,buf);
      tmp[j++] = c;
    }
  }
  SortString(tmp,j);
  deleteAllItemsOfXmList(w);
  for(i=0;i<j;i++){
    PutTextInListWidget1(w,tmp[i]);
    delete tmp[i];
  }
  delete tmp;
}
void Scheduler::addObjToMovie(RunBase *rb){
  if(activeMovie == NULL){
    activeMovie = new RunMovie(XET->Dpy,XET->drawingArea);
    activeMovie->setName("NoName");
  }
  if(rb)
    activeMovie->addRunObject(rb);
}
void Scheduler::addObjToMovie(char *key){
  RunBase *ro = findRunObject(key);
  if(ro)
    addObjToMovie(ro);
  else
    insertStringtoInfo("It is impossible,Run Object <%s> not found\n",key);
}

void Scheduler::work(XtPointer xptr, XtIntervalId *){
  RunBase *ro = (RunBase *) xptr;
  if(ro){
    ro->nextMove();
    ro->xid = XtAppAddTimeOut(XET->scheduler->app, 
			      ro->interval,
			      (XtTimerCallbackProc) Scheduler::work, 
			      xptr);
  }else
    insertStringtoInfo("Impossible error occurred in Scheduler::work()\n");
}

void Scheduler::updateMovieList(Widget w){
  activeMovie->BuildMovieXmList(w);
}
void Scheduler::MoveMovieUp(int which){
  if(activeMovie)
    activeMovie->moveUp(which);
}
void Scheduler::MoveMovieDown(int which){
  if(activeMovie)
    activeMovie->moveDown(which);
}
void Scheduler::MoveMovieToLast(int which){
  if(activeMovie)
    activeMovie->moveToLast(which);
}
void Scheduler::MoveMovieToFirst(int which){
  if(activeMovie)
    activeMovie->moveToFirst(which);
}
void Scheduler::MovieDelete(int which){
  if(activeMovie)
    activeMovie->Delete(which);
}
void Scheduler::MovieDeleteAll(){
  if(activeMovie)
    delete activeMovie;
  activeMovie  = NULL;
}
void Scheduler::deleteActiveMovie(){
  MovieDeleteAll();
}

// extract an object in the activeMovie and put it into the regular scheduler
void Scheduler::extractMovieObject(int which){
  if(activeMovie){
    RunBase *rb =  activeMovie->getAnObject(which);
    if(rb){
      RunBase *ronew = rb->Copy();
      char *name = rb->getName();
      char buff[200];
      sprintf(buff,"%s_%d",name,dup_index++);
      ALLOC_STRCPY(ronew->name,buff);
      ronew->setTermCriteria(NOT_APPLICATBLE_E,NULL);
      addRunObject(ronew->name,ronew,2);
    }
  }
}
RunBase *Scheduler::getOneMovieObject(int which){
  if(activeMovie)
    return activeMovie->getAnObject(which);
  else
    return NULL;
}
int Scheduler::MovieCommit(char *s){
  if(activeMovie){
    RunBase *ro = findRunObject(s);
    if(!ro){
      addRunObject(s,activeMovie,0,2);
      activeMovie = NULL;
      ro = findRunObject(s);
      ro->reset();
      return 1;
    }else{
      return 0;
    }
  }
}
void Scheduler::setActiveMovie(RunMovie *rb){
  if(activeMovie)
    delete activeMovie;
  activeMovie = rb;
}






