//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "textImage.h"
#include "util.h"
#include <string.h>
#include <libc.h>

TextImage::TextImage(Display *d,Widget w,GFont *f)
: Clip(d,w),gFont(f)
{
  image = NULL;
}
TextImage::~TextImage() {
  if(image){
    XDestroyImage(image);
  }
  deallocateString();
}
Clip *TextImage::dup(){
  TextImage *rtn = new TextImage(*this);
  rtn->diskFileName  = NULL;
  rtn->shapeFileName = NULL;
  rtn->clipName      = NULL;
  copyto(rtn);
  rtn->RefCounter = 0;
  return rtn;
}


// justification
// (1) left-top   (2) top-center (3) right-top
// (4) left-center (5) center-center (6) right-center
// (7) left-bottom (8) bottom-center (9) right-bottom
// if w_box == 0 use font->Height
// if h_box == 0 use font->width
// if justificatio == 0 defaults to center-center
void TextImage::DrawText(char s, Widget win, 
			 int w_box, int h_box,int xloc, int yloc,
			 int justification,
			 int size, unsigned long fg, unsigned long bg,
			 XImage *img)
{
  char ccc[2];
  ccc[0] = s;
  ccc[1] = '\0';
  DrawText(ccc,win,w_box,h_box,xloc,yloc,justification,size,fg,bg,img);
}

void TextImage::DrawText(char *s, Widget win, 
			 int w_box, int h_box,int xloc, int yloc,
			 int justification,
			 int size, unsigned long fg, unsigned long bg,
			 XImage *img){
  XFontStruct *xfs;
  xfs = gFont->LoadQueryScalableFont(size);
  if(xfs && gc)
    XSetFont(Dpy,gc,xfs->fid);
  else{
    printf("can not find the desired fontor unspecified gc\n");
    return;
  }
  int font_height = gFont->Height(s);
  int font_width = gFont->Width(s);
  Pixmap pix;
  if(img){
    height = img->height;
    width  = img->width;
    pix = XCreatePixmap(Dpy,XtWindow(win),width,height,8);
    XSetForeground(Dpy,gc,fg);
    XPutImage(Dpy,pix,gc,img,0,0,0,0,width,height);
  }else{
    height = (h_box < font_height) ? font_height : h_box;
    width  = (w_box < font_width)  ? font_width  : w_box;
    pix = XCreatePixmap(Dpy,XtWindow(win),width,height,8);
    XSetForeground(Dpy,gc,bg);
    XFillRectangle(Dpy,pix,gc,0,0,width,height);
    XSetForeground(Dpy,gc,fg);
    XSetBackground(Dpy,gc,bg);
  }
  int x_loc,y_loc;
  switch(justification){
  case 1: // top-left
    x_loc = 0;
    y_loc = xfs->ascent;
    break;
  case 2: // top-center
    x_loc = (width - font_width)/2;
    y_loc = xfs->ascent;
    break;
  case 3: // top-right
    x_loc = width - font_width;
    y_loc = xfs->ascent;
    break;
  case 4: // left-center
    x_loc = 0;
    y_loc = (height - font_height) / 2 + xfs->ascent;
    break;
  default:
  case 5: // center-center
    x_loc = (width - font_width)/2;
    y_loc = (height - font_height) / 2 + xfs->ascent;
    break;
  case 6: // right-center
    x_loc = width - font_width;
    y_loc = (height - font_height) / 2 + xfs->ascent;
    break;
  case 7: // top-left
    x_loc = 0;
    y_loc = height - font_height + xfs->ascent;
    break;
  case 8: // top-center
    x_loc = (width - font_width) / 2;
    y_loc = height - font_height + xfs->ascent;
    break;
  case 9: // top-right
    x_loc = width - font_width;
    y_loc = height - font_height + xfs->ascent;
    break;
  case 10: // fixed location
    x_loc = xloc;
    y_loc = yloc;
    break;
  case 11: // random location
    x_loc = rand() % width;
    y_loc = rand() % height;
    break;
  }
  XDrawString(Dpy,pix,gc,x_loc,y_loc,s,strlen(s));
  image = XGetImage(Dpy,pix,0 ,0,width,height,0xff,ZPixmap);
  XFreePixmap(Dpy,pix);

}


