//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#ifndef XET_H
#define XET_H

#include "WCLiptr.h"
#include "importImage.h"
#include "locus.h"
#include "font.h"
#include "run_base.h"
#include "clipsTable.h"
#include "locusTable.h"
#include "scheduler.h"
#include "outputWindow.h"

enum DrawingStatus {Ready_E, DefineLocus_E, DefineBackground_E, 
                    ReadXetFile_E, LoadXetFile_E, 
                    writeClip_E, ImportImage_E,};
class Clip;
class ProcSpooler;

class Xet  : public WCLInterpreter {
friend Scheduler;
friend RunBase;
friend RunFly;
friend RunLocus;

protected:
  Clip         *colorPallet;
  Cursor        saveCursor,loadCursor,writeCursor,importCursor;
  Display      *Dpy;
  DrawingStatus drawingStatus;
  GC            gc, locus_gc, font_gc;
  XGCValues     vals;
  Locus        *activeLocus;
  Scheduler    *scheduler;
  ClipsTable   *clipsPool;
  LocusTable   *locusPool;
  ProcSpooler  *procSpooler;
  GFont        *GlobalFont;
  Widget        mainWindow,  infoTXT, xetLogo;
  OutputWindow *drawingArea;
  OutputWindow *rootArea;
  OutputWindow *logoArea;
  ClipsTable   *logoClips;
  LocusTable   *logolocus;
  Scheduler    *logoScheduler;
  char         *flipOrientation;
  int           activeRectON;
  int           paste_x,paste_y;
  int           arx1,arx2,ary1,ary2;
  int           overrideFlag;  // == 1 override otherwise no
public:
  Xet(char *resName, Cardinal argc, char* argv[]);
  ~Xet();
  Pixmap  creatRectMask(int width,int height,int x_orig,int y_orig,
			int raduis_x,int raduis_y,ShapeMode shapeMode);
  Pixmap  creatArcMask(int width,int height,int x_orig,int y_orig,
		       int raduis_x,int raduis_y,ShapeMode shapeMode);
  Pixmap  creatStarMask(int width,int height,int,int,int,int,int,int,
		       int,int,ShapeMode shapeMode);
  Pixmap  creatGearMask(int width,int height,int,int,int,int,int,int,
		       int,int,ShapeMode shapeMode);
  Pixmap  creatSprayMask(int width,int height,double pcnt,int psz,
			 ShapeMode shapeMode);
  Pixmap  creatDiagonalMask(int width,int height,int x_orig,int y_orig,
			    int angle,int span, ShapeMode shapeMode);
  Pixmap  creatPolygonMask(int width,int height,int,int,int,int,
			   int,int,ShapeMode shapeMode);
  Pixmap  creatMeshMask(int width, int height,int,int,int,int,int,
			   int,int,ShapeMode shapeMode);
  Widget  createAreYouSureDialog(char *s); 
  ProcSpooler *getProcSpooler();
  Scheduler   *getScheduler();
  int	  DrawRect(int x,int y,int x1,int y1);
  int	  buildLocusPoints(LocusPoint *lp,int npoints,char *asciipoints);
  void	  FSBUpdateImportFilename();
  void	  MakeItOval(char *s);
  void	  ReInitialize();
  void	  actionDispatcher(Widget , char *s);
  void	  bgroundDoit();
  void	  buildClipList();
  void	  buildClipList(Widget XmListWidget);
  void	  buildSchedulerList(char *);
  void	  buildSpoolList();
  void    buildXetLogo();
  void    buildXetLogo1();
  void    BuildXetLogoMovingObjects();
  void	  clearDrawingArea();
  void    cleaninfotxt();
  void	  clipAttribApply();
  void	  cliprehearse(int flag);
  void	  clipstoprehearse();
  void	  cmapfilenameTBL();
  void    colorFreeCells();
  void	  colorSelPaste(int);
  void	  colorSelectPanel();
  void	  configClipsClone();
  void	  configClipsDelete();
  void	  configClipsDeleteAll();
  void	  configClipsDeleteMask();
  void	  configClipsDup();
  void	  configClipsInfo(char *);
  void	  configClipsQuantize();
  void	  configClipsRename();
  void	  configClipsSetStatus();
  void	  configClipsShow();
  void	  configClipsShowAll();
  void	  configENV();
  void	  configENVApply();
  void    createSeveralPipmapCursors();
  void	  cropEntireBackground();
  void	  cropTheArea();
  void	  cutTheArea();
  void    destroyAreYouSureDialog();
  void	  destroyBackground();
  void	  drawGrid(int space);
  void    dumpinfotxt();
  void	  editLocusAccept();
  void	  editLocusPoints();
  void	  editLocusUnique();
  void	  forgeApply();
  void    freezeLogo();
  void	  freezeMotion();
  void	  genericCancel(char *);
  void	  getCurrentDrawingAreaGeometry();
  void	  importDoit();
  void	  importfetchinfo();
  void	  loadcmap();
  void	  loadlocus();
  void	  loadppm(char *);
  void	  loadxetfile();
  void	  loadxetfile(char *);
  void	  locusAccept();
  void	  locusCancel();
  void	  locusClear();
  void	  locusDefine();
  void	  locusDelete();
  void	  locusDeleteAll();
  void	  locusDup();
  void	  locusIgnore();
  void	  locusInfo(char *);
  void    locusPaste();
  void	  locusRename();
  void	  locusShow();
  void	  movieCommit();
  void	  movieDelete();
  void    movieExtract();
  void	  movieDeleteAll();
  void	  movieMoveDown();
  void	  movieMoveToFirst();
  void	  movieMoveToLast();
  void	  movieMoveUp();
  void    paintDrawingArea(char *,int,int,int,int,int,int,OutputWindow *);
  void    parseAndBuildClip(char *nextField);
  void    parseAndBuildClipMask(char *nextField);
  void    parseAndBuildCloneClip(char *nextField);
  void    parseAndBuildImportClip(char *nextField);
  void    parseAndBuildLocus(char *nextField);
  void    parseAndBuildScheduler(char *nextField,char *);
  void    parseAndBuildSnapshot(char *nextField);
  void    parseAndBuildText(char *nextField);
  void    parseAndBuildbackground(char *nextField);
  void    parseAndBuildPaintDrawingArea(char *nextField);
  void	  pastePoints();
  void	  popupModalInfoBox(char *s);
  void	  popupModalInfoBox(char *,char *);
  void	  popupModalInfoBox(char *,int );
  void	  popupSchedulerEditing(char *, RunBase  *cat,int flag = 0);
  void    popupselectClipsTBL();
  void	  preWriteClipImage();
  void	  preWriteClipMask();
  void	  preschedulerWrite();
  void	  preshapeImportMask();
  void	  prewriteLocus();
  void    processCommandLine(Cardinal argc, char* argv[]);
  void    reCaptureRootWindow();
  void	  rePaintBackground();
  void    rePaintRootArea();
  void	  rePaintColorPallet();
  void    rePaintXetLogo();
  void    readForkResult(XtPointer x,int fd, XtInputId *xid);
  void	  readImagefile(char *s, char *id, int flag,ClipsTable *);
  void	  readMaskfile(char *s, char *id,ClipsTable *);
  void	  renameClipOk();
  void	  renameLocusOk();
  void	  renameSchedOk();
  void    runFreeze();
  void	  runReset();
  void	  runReverse();
  void	  runReverseClip();
  void    runUnfreeze();
  void	  runUpdate();
  void    schedOutputDrawingArea();
  void    schedOutputRoot();
  void	  schedulerDelete();
  void	  schedulerDeleteAllItems();
  void	  schedulerDuplicate();
  void	  schedulerEdit(int flag = 0);
  void	  schedulerFreeze();
  void	  schedulerMoveToMovie();
  void	  schedulerRename();
  void	  schedulerReset();
  void	  schedulerResetAll();
  void	  schedulerReverse();
  void	  schedulerReverseAll();
  void	  schedulerReverseClip();
  void	  schedulerReverseClipAll();
  void	  schedulerUnfreeze();
  void    schedulerToRootWinAll();
  void    schedulerToDAALL();
  void	  selectthisclip();
  void	  setClipStatus(ClipStatus);
  void    setEffectiveCursor(Cursor);
  void	  setFlipOrientation(char *s);
  void    setOverrideFlag(int);
  void    setSystemDefinedCursor(int);
  void	  setclipAttributes();
  void	  shapeDoit();
  void	  shapeImportMask();
  void	  spoolerClose();
  void	  spoolerDelete();
  void	  spoolerDeleteAll();
  void	  spoolerMoveDown();
  void	  spoolerMoveUp();
  void	  spoolerResume();
  void	  spoolerSuspend();
  void	  textDoit();
  void    unfreezeLogo();
  void	  unfreezeMotion();
  void	  updateColorPanelFields(XEvent *evt);
  void	  updateMovieList();
  void	  updateSpoolerPanel();
  void	  updateSpoolerStatus();
  void	  updateXetStatistic();
  void	  updatedrawingWhereField(XEvent *evt);
  void	  writeBackgroundImage(char *);
  void	  writeClipImage();
  void	  writeClipMask();
  void	  writeCmapFileAs();
  void	  writeLocus();
  void	  writeOutXetFile(char *filename);
  void	  writeOutXetFileAs();
  void	  writeScheduler();
  void	  xetHelp(char *);

};

#endif



