//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include "config.h"
#include "xet.h"
#include <stdio.h>
#include <libc.h>
#include <osfcn.h>
#include <string.h>
#include <Xm/FileSB.h>
#include <Xm/TextF.h>
#include <X11/cursorfont.h>
#include "ppmImage.h"
#include "textImage.h"
#include "cutbgImage.h"
#include "cloneImage.h"
#include "wrapper.h"
#include "util.h"
#include "run_fly.h"
#include "run_glide.h"
#include "run_random.h"
#include "run_locus.h"
#include "run_fixed.h"
#include "run_blink.h"
#include "run_movie.h"
#include "color.h"
#include "clipList.h"
#include "colorImage.h"

extern Color      *currentCmap;

static char        loadFromThisDirectory[256];
static int         inputLineNo;
static RunMovie   *activeRunMovie;
static int         buildingAnimatedLogo;

static BoundaryMode cvtToBoundaryMode(char *p){
  if(strcmp(p, "Wrap") == 0)
  return  Wrap_E;       
  if(strcmp(p, "BounceBack") == 0)
  return  BounceBack_E; 
  if(strcmp(p, "Disappear") == 0)
  return  Disappear_E;
  insertStringtoInfo("Unrecognized boundary mode, default to wrap mode\n");
  return Wrap_E;
}
static ShapeMode cvtToShapeMode(char *p){
  if(strcmp(p,"hollow") == 0) return HOLLOW_E;
  if(strcmp(p,"solid") == 0)  return SOLID_E;
  return SOLID_E;
}
static ShapeType cvtToShapeType(char *p){
  if(strcmp(p,"circle") == 0)          return CIRCLE_E;      
  if(strcmp(p,"ellipse") == 0)         return ELLIPSE_E;        
  if(strcmp(p,"rectangle") == 0)       return RECTANGULAR_E; 
  if(strcmp(p,"star") == 0)            return STAR_E;       
  if(strcmp(p,"gear") == 0)            return GEAR_E;   
  if(strcmp(p,"spray") == 0)           return SPRAY_E;
  if(strcmp(p,"mesh") == 0)            return MESH_E;  
  if(strcmp(p,"diagonal") == 0)        return DIAGONAL_E;    
  if(strcmp(p,"polygon") == 0)         return POLYGON_ES_E;    
  return CIRCLE_E; 
}
static int cvtToTileMode(char *p){
  if(strcmp(p,"tiled") == 0)         return 1;
  if(strcmp(p,"untiled") == 0)       return 2;
  if(strcmp(p,"tiledV") == 0)        return 3;
  if(strcmp(p,"tiledH") == 0)        return 4;
  return 1;
}
static int cvtToROMode(char *p){
  if(strcmp(p,"replace") == 0)       return 1;
  if(strcmp(p,"override") == 0)      return 2;
  return 1;
}
static int cvtToTextAdjustment(char *p){
  if(strcmp(p,"left_top") == 0)       return 1;
  if(strcmp(p,"center_top") == 0)     return 2;
  if(strcmp(p,"right_top") == 0)      return 3;
  if(strcmp(p,"left_center") == 0)    return 4;
  if(strcmp(p,"center_center") == 0)  return 5;
  if(strcmp(p,"right_center") == 0)   return 6;
  if(strcmp(p,"left_bottom") == 0)    return 7;
  if(strcmp(p,"center_bottom") == 0)  return 8;
  if(strcmp(p,"right_bottom") == 0)   return 9;
  if(strcmp(p,"fixed_location") == 0) return 10;
  if(strcmp(p,"random_location") == 0) return 11;
  return 5;
}
static TrailingMode cvtToTrailingMode(char *p){
  if(strcmp(p, "Preserve") == 0)
    return Preserve_E; 
  if(strcmp(p, "Wipeout") == 0)
    return Wipeout_E;  
  if(strcmp(p, "Leave") == 0)
    return  Leave_E;    
  if(strcmp(p, "Rainbow") == 0)
    return  Rainbow_E;  
  insertStringtoInfo("Unrecognized trailing mode, default to preserve mode\n");
  return Preserve_E;  
}

static char *fieldSkip(char *p)
{
  //Search for end of field.
  
  while( *p && *p != ':' && *p != '\n' )
    ++p;
  
  //Make sure field is terminated by a null character.
  if( *p ) *p++ = 0;
  
  //Return pointer to beginning of next field, or null character
  //if we have just skipped the last field.
  return(p);
}
void Xet::setOverrideFlag(int i){
  overrideFlag = i;
}

void Xet::loadxetfile(){
  Widget filer             = nameToWidget("LoadFSB*loadfsb");
  Widget selectedText      = XmFileSelectionBoxGetChild(filer,XmDIALOG_TEXT);
  char *fileName           = XmTextGetString(selectedText);
  loadxetfile(fileName);
  XtFree(fileName);
}
void Xet::loadxetfile(char *fileName){
  insertStringtoInfo("Start loading xet script <%s>.\n",fileName);
  inputLineNo = 0;
  buildingAnimatedLogo = 0;
  activeRunMovie = NULL;
  FILE *fp = fopen(fileName,"r");
  if(fp == NULL){
    popupModalInfoBox("Can not open file <%s> \n",fileName);
    XtFree(fileName);
    return;
  }
  setEffectiveCursor(loadCursor);
  //find out the location of the last \'/\' character
  int i,len = strlen(fileName);
  for(i=len-1;i>=0;i--){
    if(fileName[i] == '/')
      break;
  }
  strncpy(loadFromThisDirectory,fileName,i);
  loadFromThisDirectory[i] = '\0';
  char line[BUFSIZ+1];
  char *p;
  char *nextField;
  char errbuf[200];
  int howManyUnknownActions = 0;
  for(;;){
    if((p = fgets(line, BUFSIZ, fp)) == NULL)
      break;     // end of file
    inputLineNo++;
    if(line[0] == '#')
      continue;  // hit comments
    if(isBlankString(p) == 1)
      continue;
    nextField = fieldSkip(p);
    if(strcmp(p,"StartAnimatedLogo") == 0){
      buildingAnimatedLogo = 1;
      continue;
    }
    if(strcmp(p,"EndOfAnimatedLogo") == 0){
      buildingAnimatedLogo = 0;
      continue;
    }
    if(strcmp(p,"background") == 0){
      parseAndBuildbackground(nextField);
      continue;
    }
    if(strcmp(p,"paintdrawingarea") == 0){
      parseAndBuildPaintDrawingArea(nextField);
      continue;
    }
    if(strcmp(p,"clip") == 0){
      parseAndBuildClip(nextField);
      continue;
    }
    if(strcmp(p,"cloneclip") == 0){
      parseAndBuildCloneClip(nextField);
      continue;
    }
    if(strcmp(p,"locus") == 0){
      parseAndBuildLocus(nextField);
      continue;
    }
    if(strcmp(p,"scheduler") == 0){
      parseAndBuildScheduler(nextField,"scheduler");
      continue;
    }
    if(strcmp(p,"movie") == 0){
      parseAndBuildScheduler(nextField,"movie");
      continue;
    }
    if(strcmp(p,"text") == 0){
      parseAndBuildText(nextField);
      continue;
    }
    if(strcmp(p,"snapshot") == 0){
      parseAndBuildSnapshot(nextField);
      continue;
    }
    if(strcmp(p,"clipmask") == 0){
      parseAndBuildClipMask(nextField);
      continue;
    }
    if(strcmp(p,"importclip") == 0){
      parseAndBuildImportClip(nextField);
      continue;
    }
    if(strcmp(p,"end-of-movie") == 0){
      if(activeRunMovie)
	activeRunMovie->reset();
      activeRunMovie = NULL;
      continue;
    }
    howManyUnknownActions++;
    if(howManyUnknownActions > 10){
      insertStringtoInfo("Too much unknown actions, are you sure it is an Xet file ?\n");
      goto D99;
    }
    sprintf(errbuf,"Unknown action <%s> at line <%d> in file <%s>\n",p,
	    inputLineNo,fileName);
    insertStringtoInfo(errbuf);
  }
  insertStringtoInfo("Finish loading xet script <%s>.\n",fileName);
 D99:
  fclose(fp);
  setSystemDefinedCursor(XC_left_ptr);
  XFlush(Dpy);
}
void Xet::parseAndBuildLocus(char *nextField){
  char *p;
  char *locusName = nextField;
  p = fieldSkip(nextField);
  char *fileName = p;
  p = fieldSkip(p);
  char *locuszise = p;
  p = fieldSkip(p);
  int  size = atoi(locuszise);
  Locus *l = locusPool->findLocus(locusName); 
  if(l != NULL){
    if(overrideFlag != 1){
      insertStringtoInfo("Locus <%s> already exist, ignore the new one\n",locusName);
      return;
    }else{
      locusPool->deleteLocus(locusName);
    }
  }
  Locus *locus = new Locus(10);
  char buffer[256],khn[256];
  sprintf(buffer,"%s/%s",loadFromThisDirectory,fileName);
  if(locus->load(buffer) == 1){
    locus->setName(locusName);
    locusPool->addLocus(locusName,locus);
    sprintf(khn,"Loading locus <%s> from file <%s> successful.\n",locusName,buffer);
  }else{
    sprintf(khn,"Loading locus <%s> from file <%s> failed.\n",locusName,buffer);
  }
  insertStringtoInfo(khn);
}

void Xet::parseAndBuildSnapshot(char *nextField){
  char *p;
  char *clipName = nextField;
  p = fieldSkip(nextField);
  char *region = p;
  p = fieldSkip(p);
  int x,y,width,height;
  sscanf(region,"%d %d %d %d",&x,&y,&width,&height);
  Clip *clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(clipName);
  if(clp != NULL){
    if(overrideFlag != 1){
      insertStringtoInfo("Clip <%s> already exist, ignore the new one\n",clipName);
      return;
    }else
      ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->deleteClip(clipName);
  }
  clp = new CutbgImage(Dpy,drawingArea->da_area,drawingArea->backgroundImage);
  clp->MakeImage(width,height,x,y,0,0);
  clp->SetGC(gc);
  clp->setClipName(clipName);
  ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->addClip(clipName,clp);
  char khn[256];
  sprintf(khn,"Build snapshot clip <%s> by cutting (%d %d %d %d) from drawing area.\n",
	  clipName,x,y,width,height);
  insertStringtoInfo(khn);
}

void Xet::parseAndBuildClipMask(char *nextField){
  char *p;
  char *clipName = nextField;
  p = fieldSkip(nextField);
  char *mode = p;
  p = fieldSkip(p);
  char *type = p;
  p = fieldSkip(p);
  Clip *clp;
  clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(clipName);
  if(clp == NULL){
    insertStringtoInfo("Clip <%s> doesn't exist, can not attach mask to it\n",clipName);
    return;
  }
  if(clp->getShapeMask() && overrideFlag != 1){
    insertStringtoInfo("Clip <%s> already has a mask, ignore the new one\n",clipName);
    return;
  }
  char khn[256];
  if(strcmp(mode,"import") == 0){    // must read from a mask file
    char buffer[100];
    sprintf(buffer,"%s/%s",loadFromThisDirectory,type);
    readMaskfile(buffer,clipName,
		 (buildingAnimatedLogo == 1) ? logoClips : clipsPool);
    sprintf(khn,"Import mask for clip <%s> from <%>\n",clipName,buffer);
    insertStringtoInfo(khn);
    return;
  }
  char *xorig = p;    int x_orig = atoi(xorig);
  p = fieldSkip(p);
  char *yorig = p;    int y_orig = atoi(yorig);
  p = fieldSkip(p);
  char *parm_1 = p;   int p1 = atoi(parm_1);
  p = fieldSkip(p);
  char *parm_2 = p;   int p2 = atoi(parm_2);
  p = fieldSkip(p);
  char *parm_3 = p;   int p3 = atoi(parm_3);
  p = fieldSkip(p);
  char *parm_4 = p;   int p4 = atoi(parm_4);
  p = fieldSkip(p);
  char *parm_5 = p;   int p5 = atoi(parm_5);
  p = fieldSkip(p);
  char *parm_6 = p;   int p6 = atoi(parm_6);
  p = fieldSkip(p);
  double dp1 = atof(yorig);
  ShapeMode shapeMode = cvtToShapeMode(mode);
  ShapeType shapeType = cvtToShapeType(type);
  Pixmap pix;
  switch(shapeType){
  case ELLIPSE_E:
    pix = creatArcMask(clp->Width(),clp->Height(),x_orig,y_orig,p1,p2,
		       shapeMode);
    break;
  case CIRCLE_E:
    pix = creatArcMask(clp->Width(),clp->Height(),x_orig,y_orig,p1,p1,
		       shapeMode);
    break;        
  case RECTANGULAR_E:
    pix = creatRectMask(clp->Width(),clp->Height(),x_orig,y_orig,
			p1,p2,shapeMode);
    break; 
  case STAR_E:
    pix = creatStarMask(clp->Width(),clp->Height(),x_orig,y_orig,
			p1,p2,p3,p4,p5,p6,shapeMode);
    break;       
  case GEAR_E:
    pix = creatGearMask(clp->Width(),clp->Height(),x_orig,y_orig,
			p1,p2,p3,p4,p5,p6,shapeMode);
    break;     
  case SPRAY_E:
    pix = creatSprayMask(clp->Width(),clp->Height(),dp1,x_orig,
			 shapeMode);
    break;
  case MESH_E:
    // for some reason oslx and osly is reversed, islx and isly is also
    // reverse, instead of changing the creatMeshMask code I simply changed
    // the order of the oslx/osly and islx/isly.
    pix = creatMeshMask(clp->Width(),clp->Height(),x_orig,y_orig,
			p2,p1,p4,p3,p5,shapeMode);
    break;  
  case DIAGONAL_E:
    if(p1 <= 180 && p1 >= 0){
      switch(p1){
      case 90:
	pix = creatRectMask(clp->Width(),clp->Height(),x_orig,y_orig,
			    p2,clp->Height(),shapeMode);
	break;
      case 0:
      case 180:
	pix = creatRectMask(clp->Width(),clp->Height(),x_orig,y_orig,
			    clp->Width(),p2,shapeMode);
	break;
      default:
	pix = creatDiagonalMask(clp->Width(),clp->Height(),x_orig,y_orig,
				p1,p2, shapeMode);
      }
    }else
      insertStringtoInfo("Orientation angle must be between 0 and 180 degree\n");
    break;    
  case POLYGON_ES_E:
    pix = creatPolygonMask(clp->Width(),clp->Height(),x_orig,y_orig,
			   p1,p2,p3,p4, shapeMode);
    break;
  }
  clp->setShapeMask(pix);
  insertStringtoInfo("A clip mask is attached to clip <%s>\n",clipName);
}

void Xet::parseAndBuildImportClip(char *){
  // this function is not necessary
  // loading xetfile to an Xet session should be done is a short period of time
  // Therefore, it shouldn\'t have any image transformation involved
}

void Xet::parseAndBuildText(char *nextField){
  char *p;
  char *clipName = nextField;
  p = fieldSkip(nextField);
  char *textbground = p;
  p = fieldSkip(p);
  char *textfgcolor = p;
  p = fieldSkip(p);
  char *bgClipName = p;
  p = fieldSkip(p);
  char *clipSize = p;
  p = fieldSkip(p);
  char *fontType = p;
  p = fieldSkip(p);
  char *fontSize = p;
  p = fieldSkip(p);
  char *textString = p;
  p = fieldSkip(p);
  char *justification = p;
  p = fieldSkip(p);
  char *x_loc = p;
  p = fieldSkip(p);
  char *y_loc = p;
  p = fieldSkip(p);
  Clip *clp;
  clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(clipName);
  if(clp != NULL){
    if(overrideFlag != 1){
      insertStringtoInfo("Clip <%s> already exist, ignore the new one\n",clipName);
      return;
    }else{
      ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->deleteClip(clipName); 
    }
  }
  int textAdj = cvtToTextAdjustment(justification);
  if(textAdj == 0){
    insertStringtoInfo("Unknow text justification <%s>, default to center-center\n",
		       justification);
    textAdj = 5;
  }
  int fsz = atoi(fontSize);
  int flag;
  if(strcmp("",fontType) == 0)
    flag = GlobalFont->SetFont("-bitstream-courier-bold-*-*-*-0-0-*-*-*-0-*-*");
  else
    flag = GlobalFont->SetFont(fontType);
  if(flag == 0){
    insertStringtoInfo("<%s> is not a valid font\n",fontType);
    return;
  }
  int font_width,font_height;
  if(strcmp("",clipSize) == 0){
    XFontStruct *xfs = GlobalFont->LoadQueryScalableFont(fsz);
    if(xfs){
      font_width = GlobalFont->Width(textString);
      font_height = GlobalFont->Height(textString);
    }else{
      insertStringtoInfo("<%s> is not a valid font\n",fontType);
      return;
    }
  }else{
    sscanf(clipSize,"%d x %d",&font_width,&font_height);
  }
  unsigned long fg,bg;
  if(strcmp("",textbground) != 0)
    bg = atoi(textbground);
  else
    bg = WhitePixel(Dpy,DefaultScreen(Dpy));
  if(strcmp("",textfgcolor) != 0)
    fg = atoi(textfgcolor);
  else
    fg = BlackPixel(Dpy,DefaultScreen(Dpy));
  XImage *img;
  if(strcmp("",bgClipName) == 0)
    img = NULL;
  else{
    Clip *clp;
    clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(bgClipName);
    if(clp == NULL){
      insertStringtoInfo("Clip <%s> doesn't exist. assume text doesn't use background image\n",bgClipName);
      img = NULL;
    }else
      img = clp->GetImage();
  }
  int xloc = atoi(x_loc);
  int yloc = atoi(y_loc);
  TextImage *textImage;
  if(GlobalFont->IsScalableFont(fontType) == 0 || fsz != 0){
    textImage = new TextImage(Dpy,drawingArea->da_area,GlobalFont);
    textImage->SetGC(font_gc);
    textImage->setClipName(clipName);
    textImage->DrawText(textString,drawingArea->da_area,font_width,font_height,xloc,yloc,
			textAdj,fsz,fg,bg,img);
    ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->addClip(clipName,textImage);
    char khn[256];
    sprintf(khn,"Build text clip <%s> size = (%d x %d) font size %d type = %s\n",
	    clipName,font_width,font_height,fsz,fontType);
    insertStringtoInfo(khn);
  }else
    insertStringtoInfo("Build text clip <%s> failed. Font size must be greater then 0 for a scaleable font\n",clipName);
  
}

void Xet::parseAndBuildClip(char *nextField){
  char *p;
  char *clipName = nextField;
  p = fieldSkip(nextField);
  char *fileName = p;
  p = fieldSkip(p);
  char *shapefilename = p;
  p = fieldSkip(p);
  Clip *clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(clipName);
  if(clp != NULL){
    if(overrideFlag != 1){
      insertStringtoInfo("Clip <%s> already exist, ignore the new one\n",clipName);
      return;
    }else
      ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->deleteClip(clipName);
  }
  char buffer[256];
  sprintf(buffer,"%s/%s",loadFromThisDirectory,fileName);
  readImagefile(buffer,clipName,0,
		(buildingAnimatedLogo == 1) ? logoClips : clipsPool);
  if(strcmp("",shapefilename) != 0){
    sprintf(buffer,"%s/%s",loadFromThisDirectory,shapefilename);
    readMaskfile(buffer,clipName,
		 (buildingAnimatedLogo == 1) ? logoClips : clipsPool);
  }
  char khn[256];
  sprintf(khn,"build clip <%s> from <%s> and mask form <%s>\n",
	  clipName,fileName,shapefilename);
  insertStringtoInfo(khn);
}

void Xet::parseAndBuildCloneClip(char *nextField){
  char *p;
  char *clipName = nextField;
  p = fieldSkip(nextField);
  char *origclipName = p;
  p = fieldSkip(p);
  char *shapefilename = p;
  p = fieldSkip(p);

  Clip *clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(clipName);
  if(clp != NULL){
    if(overrideFlag != 1){
      insertStringtoInfo("Clip <%s> already exist, ignore the new one\n",clipName);
      return;
    }else
      ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->deleteClip(clipName);
  }
  Clip *origclp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(origclipName);
  if(origclp == NULL){
    insertStringtoInfo("Clip <%s> not found\n",origclipName);
    return;
  }
  Clip *newclip = new CloneImage(origclp);
  newclip->setClipName(clipName);
  ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->addClip(clipName,newclip);
  char buffer[256];
  if(strcmp("",shapefilename) != 0){
    sprintf(buffer,"%s/%s",loadFromThisDirectory,shapefilename);
    readMaskfile(buffer,clipName,
		 (buildingAnimatedLogo == 1) ? logoClips : clipsPool);
  }
  char khn[256];
  sprintf(khn,"build a clone clip <%s> from <%s> mask file <%s>\n",
	  clipName,origclipName,shapefilename);
  insertStringtoInfo(khn);

}

void Xet::parseAndBuildPaintDrawingArea(char *nextField){
  char *p;
  char *clipName = nextField;
  p = fieldSkip(nextField);
  char *pix = p;
  p = fieldSkip(p); 
  char *destination = p;
  p = fieldSkip(p);  
  char *ormode = p;
  p = fieldSkip(p);
  char *tilingmode = p;
  p = fieldSkip(p);
  char *justification = p;
  p = fieldSkip(p);
  int x_loc = atoi(p);
  p = fieldSkip(p);
  int y_loc = atoi(p);
  p = fieldSkip(p);
  int pixval;
  if(strcmp("",pix) == 0)
    pixval = -1;
  else
    pixval = atoi(pix);
  Clip *clp = ((buildingAnimatedLogo == 1) ? logoClips : clipsPool)->findClip(clipName);
  if(clp == NULL && strcmp("",clipName) != 0){
    insertStringtoInfo("Clip <%s> not found can not paint background.\n",clipName);
    return;
  }
  int overrideMode = cvtToROMode(ormode);
  int justMode     = cvtToTextAdjustment(justification);
  int tileMode     = cvtToTileMode(tilingmode);
  OutputWindow *outputWidget = drawingArea;
  if(buildingAnimatedLogo == 1){
    outputWidget = logoArea;
  }else{
    if(strcmp(destination,"root") == 0)
      outputWidget = rootArea;
    else{
      if(strcmp(destination,"drawingarea") == 0 || strcmp(destination,"") == 0)
	outputWidget = drawingArea;
      else{
	if(strcmp(destination,"logoarea") == 0)
	  outputWidget = logoArea;
      }
    }
  }
  paintDrawingArea(clipName,pixval,overrideMode,tileMode,justMode,
		   x_loc,y_loc,outputWidget);
  char khn[256];
  sprintf(khn,"Paint drawing area with clip <%s> as <%s,%s,%s,%s> \n",
	  clipName,destination,ormode,tilingmode,justification);
  insertStringtoInfo(khn);
  
}
void Xet::parseAndBuildbackground(char *nextField){
  char *bgfile = nextField;
  char *p = fieldSkip(nextField);
  char buffer[100];
  sprintf(buffer,"%s/%s",loadFromThisDirectory,bgfile);
  PPMImage *ppmimage;
  currentCmap->Initialize();
  ppmimage = new PPMImage(Dpy,drawingArea->da_area,buffer);
  ppmimage->SetGC(gc); 
  if(ppmimage->loadPPM() == -1){
    insertStringtoInfo("Bad ppm file <%s>, can not paint drawing area.\n",buffer);
  }else{
    insertStringtoInfo("Fitting %d colors into default color map\n",
		       currentCmap->howMany());
    currentCmap->fitIntoDefaultColormap();
    ppmimage->buildXimage();
    ppmimage->setNcolors(currentCmap->howMany());
    Pixmap pix = XCreatePixmap(Dpy,drawingArea->window,
			       drawingArea->da_width,
			       drawingArea->da_height,8);
    XPutImage(drawingArea->Dpy, pix, gc, ppmimage->GetImage(), 0, 0,
	      0 , 0 , drawingArea->da_width,drawingArea->da_height);
    
    delete ppmimage;
    drawingArea->setImage(pix);
    drawingArea->putImage(gc);
    XFreePixmap(Dpy,pix);
    char khn[256];
    sprintf(khn,"build drawing area image from <%s> \n",buffer);
    insertStringtoInfo(khn);
  }
}

void Xet::parseAndBuildScheduler(char *nextField,char *what){
  if(strcmp(what,"movie") == 0 && activeRunMovie == NULL)
    return;
  char *p;
  char *runObj = p = nextField;
  p = fieldSkip(p);
  char *outputwin = p;
  p = fieldSkip(p);
  char *runMode = p;
  p = fieldSkip(p);
  char *clipName = p;
  p = fieldSkip(p);
  OutputWindow *outputWidget = drawingArea;
  if(buildingAnimatedLogo == 1){
    outputWidget = logoArea;
  }else{
    if(strcmp(outputwin,"root") == 0){
      outputWidget = rootArea;
    }else{
      if(strcmp(outputwin,"drawingarea") == 0 || strcmp(outputwin,"") == 0){
	outputWidget = drawingArea;
      }else{
	if(strcmp(outputwin,"logoarea") == 0)
	  outputWidget = logoArea;
      }
    }
  }
  RunBase      *glide = NULL;
  unsigned long interval;
  ClipList *clp;
  
  RunBase *rb;
  rb = ((buildingAnimatedLogo == 1) ? logoScheduler : scheduler)->findRunObject(runObj);
  if(rb){
    if(activeRunMovie == NULL && overrideFlag != 1){
      insertStringtoInfo("Run object <%s> already exist, ignore the new one\n",runObj);
      return;
    }else{ 
      if(activeRunMovie == NULL) 
	((buildingAnimatedLogo == 1) ? logoScheduler : scheduler)->Delete(runObj);
    }
  }
  if(strcmp(runMode,"movie") == 0){
    activeRunMovie = new RunMovie(Dpy,outputWidget);
    interval = 0;
    ((buildingAnimatedLogo == 1) ? logoScheduler : scheduler)->addRunObject(runObj,activeRunMovie,interval, 2);
    activeRunMovie->setName(runObj);
    return;
  }

  clp = new ClipList(clipName,((buildingAnimatedLogo == 1) ? logoClips : clipsPool));
  if(clp->Size() == 0){
    insertStringtoInfo("Cliplist <%s> is empty or a bad list of clip names. \n",clipName);
    return;
  }

  if(strcmp(runMode,"random") == 0){
    char *trailingMode = p;
    p = fieldSkip(p);
    char *b1 = p;
    p = fieldSkip(p);
    char *b2 = p;
    p = fieldSkip(p);
    char *b3 = p;
    p = fieldSkip(p);
    char *b4 = p;
    p = fieldSkip(p);
    interval = atoi(p);
    glide = new RunRandom(Dpy,outputWidget,clp);
    glide->start(atoi(b1),atoi(b2),atoi(b3),atoi(b4));   
    glide->setTrailingMode(cvtToTrailingMode(trailingMode));
  }
  if(strcmp(runMode,"locus") == 0){
    char *trailingMode = p;
    p = fieldSkip(p);
    char *boundaryMode = p;
    p = fieldSkip(p);
    char *locusName = p;
    p = fieldSkip(p);
    int trans_x = atoi(p);
    p = fieldSkip(p);
    int trans_y = atoi(p);
    p = fieldSkip(p);
    interval = atoi(p);
    Locus *locus = locusPool->findLocus(locusName);
    if(locus != NULL){
      glide = new RunLocus(Dpy,outputWidget,clp);
      glide->setLocus(locus);
      locus->incrRefCounter();
      glide->start(0,0,0,0);
      glide->setTranslation(trans_x,trans_y);
      glide->setBoundaryMode(cvtToBoundaryMode(boundaryMode));
      glide->setTrailingMode(cvtToTrailingMode(trailingMode));
    }else
      insertStringtoInfo("Locus <%s> doesn't exist, must be an error\n",locusName);
  }
  if(strcmp(runMode,"fly") == 0){
    char *trailingMode = p;
    p = fieldSkip(p);
    char *boundaryMode = p;
    p = fieldSkip(p);
    char *x_equ = p;
    p = fieldSkip(p);
    char *y_equ = p;
    p = fieldSkip(p);
    int fly_start = atoi(p);
    p = fieldSkip(p);
    int fly_end = atoi(p);
    p = fieldSkip(p);
    int fly_incr = atoi(p);
    p = fieldSkip(p);
    int fly_current = atoi(p);
    p = fieldSkip(p);
    interval = atoi(p);
    glide = new RunFly(Dpy,outputWidget,clp);
    glide->start(0,0,0,0);
    glide->setBoundaryMode(cvtToBoundaryMode(boundaryMode));
    glide->setTrailingMode(cvtToTrailingMode(trailingMode));
    if(glide->setFlyEquationI(x_equ,y_equ,fly_start,fly_end,fly_incr,fly_current) != 1){
      insertStringtoInfo("Syntax error on fly equation, ignore this entry\n");
      delete glide;
      delete clp;
      return;
    }
  }
  if(strcmp(runMode,"fixed") == 0){
    char *trailingMode = p;
    p = fieldSkip(p);
    int start_x = atoi(p);
    p = fieldSkip(p);
    int start_y = atoi(p);
    p = fieldSkip(p);
    interval = atoi(p);
    glide = new RunFixed(Dpy,outputWidget,clp);
    glide->start(start_x, start_y, 0,0);
    glide->setTrailingMode(cvtToTrailingMode(trailingMode));
  }
  if(strcmp(runMode,"glide") == 0){
    char *trailingMode = p;
    p = fieldSkip(p);
    int start_x = atoi(p);
    p = fieldSkip(p);
    int start_y = atoi(p);
    p = fieldSkip(p);
    int d_x = atoi(p);
    p = fieldSkip(p);
    int d_y = atoi(p);
    p = fieldSkip(p);
    int b_x = atoi(p);
    p = fieldSkip(p);
    int b_y = atoi(p);
    p = fieldSkip(p);
    int b_width = atoi(p);
    p = fieldSkip(p);
    int b_height = atoi(p);
    p = fieldSkip(p);
    interval = atoi(p);
    glide = new RunGlide(Dpy,outputWidget,clp);
    glide->start(start_x, start_y, d_x, d_y);
    glide->setBoundingBox(b_x,b_y,b_width,b_height);
    glide->setTrailingMode(cvtToTrailingMode(trailingMode));
  }
  if(strcmp(runMode,"blink") == 0){
    char *trailingMode = p;
    p = fieldSkip(p);
    int start_x = atoi(p);
    p = fieldSkip(p);
    int start_y = atoi(p);
    p = fieldSkip(p);
    interval = atoi(p);    glide = new RunBlink(Dpy,outputWidget,clp);
    glide->start(start_x, start_y, 0, 0);
    glide->setTrailingMode(cvtToTrailingMode(trailingMode));
  }
  if(glide && strcmp("",runObj) != 0){
    if(activeRunMovie == NULL){
      ((buildingAnimatedLogo == 1) ? logoScheduler : scheduler)->addRunObject(runObj,glide,interval, 2);
      glide->setName(runObj);
    }else{
      p = fieldSkip(p);
      char *termCond = p;
      p = fieldSkip(p);
      char *value = p;
      p = fieldSkip(p);
      glide->setTermCriteria(termCond,value);
      glide->interval = interval;
      glide->setName(runObj);
      activeRunMovie->addRunObject(glide);
    }
  }
  insertStringtoInfo("Build animation Object <%s> done.\n",runObj);
}

void Xet::loadppm(char *fileName){
  if(currentCmap->getCmapStatus() == DEFAULT_E)
    insertStringtoInfo("Not in PRIVATE cmap mode, but we still create the cmap\n");
  PPMImage *ppmimage;
  currentCmap->Initialize();
  ppmimage = new PPMImage(Dpy,drawingArea->da_area,fileName);
  ppmimage->SetGC(gc);
  if(ppmimage->loadPPM() == -1){
    popupModalInfoBox("Bad ppm file <%s>",fileName);
  }else{
    currentCmap->buildPrivateCmap();
  }
  delete ppmimage;
}

void Xet::loadcmap(){
  Widget filer             = nameToWidget("LoadcmapFSB*loadcmapfsb");
  Widget selectedText      = XmFileSelectionBoxGetChild(filer,XmDIALOG_TEXT);
  char *fileName = XmTextGetString(selectedText);
  FILE *cmap;
  cmap = fopen(fileName,"r");
  if(cmap){ 
    char c = getc(cmap);
    if(c == 'P' || c == 'p'){ //must be a ppm file
      fclose(cmap);
      loadppm(fileName);
      insertStringtoInfo("Loading ppm colormap from file <%s> done.\n",fileName);
    }else{
      if(isdigit(c)){
	ungetc(c,cmap);
	currentCmap->loadcmap(cmap);
	fclose(cmap);
	insertStringtoInfo("Loading ascii colormap from file <%s> done.\n",fileName);
      }else
	popupModalInfoBox("invalid colormap file");
    }
  }
  else
    insertStringtoInfo("Can not open colormap <%s>\n",fileName);


  // see if colorSelTBL is open, if it is then also force it to new colormap
  Widget colorSelTBL = nameToWidget("colorSelTBL");
  if(colorSelTBL)
    currentCmap->setWindowColormap(colorSelTBL);

  // now set the XET_CURRENT_COLOR_MAP_FILE
  ALLOC_STRCPY(XET_CURRENT_COLOR_MAP_FILE,fileName);
  XtFree(fileName);
}

void Xet::colorSelectPanel(){
  Widget colorPanelDA = nameToWidget("colorSelTBL*colorPanelDA");
  XtAddCallback(colorPanelDA ,XmNexposeCallback,
		(XtCallbackProc) PaintColorPallet,
		(caddr_t) this);
  XtAddEventHandler(colorPanelDA ,Button1MotionMask | ButtonPressMask,
		    False,
		    (XtEventHandler) updateColorPanel,
		    NULL); 
}

void Xet::rePaintColorPallet(){
  Widget colorPanelDA = nameToWidget("colorSelTBL*colorPanelDA");
  Window       da_root;       // useless variables
  int          da_x,da_y;     // useless variables
  unsigned int da_width,da_height,da_border_width,da_depth;
  XGetGeometry(XtDisplay(colorPanelDA),XtWindow(colorPanelDA),&da_root,
	       &da_x,&da_y,
	       &da_width,&da_height,&da_border_width,&da_depth);
  int xloc,yloc;
  int clip_width = colorPallet->Width();
  int clip_height = colorPallet->Height();
  if(colorPallet && colorPanelDA){
    xloc = ((da_width > clip_width) ? (da_width - clip_width) / 2 : 0);
    yloc = ((da_height > clip_height) ? (da_height - clip_height) / 2  : 0);
    OutputWindow *ow = new OutputWindow(colorPanelDA);
    colorPallet->DisplayImage(ow,xloc,yloc);
    delete ow;
    // cache a colormap so each pixel can find its rgb value
    ((ColorImage *)colorPallet)->cacheCmap();
  }

  Widget colorSelTBL = nameToWidget("colorSelTBL");
  if(colorSelTBL)
    currentCmap->setWindowColormap(colorSelTBL);

}

void Xet::writeCmapFileAs(){
  Widget filedirectoryTF  = nameToWidget("cmapfilenameTBL*filedirectoryTF");
  Widget cmapfnameppmTF   = nameToWidget("cmapfilenameTBL*cmapfnameppmTF");
  Widget cmapfnameasciiTF = nameToWidget("cmapfilenameTBL*cmapfnameasciiTF");

  char *filedirectory  = XmTextFieldGetString(filedirectoryTF);
  char *cmapfnameppm   = XmTextFieldGetString(cmapfnameppmTF);
  char *cmapfnameascii = XmTextFieldGetString(cmapfnameasciiTF);
  char buffer[200];
  if(strcmp("",cmapfnameppm) != 0){
    if(strcmp("",filedirectory) == 0){
      sprintf(buffer,"%s/%s",CURRENT_DIRECTORY,cmapfnameppm);
    }else{
      sprintf(buffer,"%s/%s",filedirectory,cmapfnameppm);
    }
    currentCmap->writeCurrentCmapInPPm(buffer);
    insertStringtoInfo("Wrote (ppm) cmap file <%s>\n",buffer);
  }
  if(strcmp("",cmapfnameascii) != 0){
    if(strcmp("",filedirectory) == 0){
      sprintf(buffer,"%s/%s",CURRENT_DIRECTORY,cmapfnameascii);
    }else{
      sprintf(buffer,"%s/%s",filedirectory,cmapfnameascii);
    }
    currentCmap->writeCurrentCmapInASCII(buffer);
    insertStringtoInfo("Wrote (ascii) cmap file <%s>\n",buffer);
  }
  if(strcmp("",cmapfnameascii) == 0 && strcmp("",cmapfnameppm) == 0)
    insertStringtoInfo("Don't know which file to write to.\n");
  XtFree(filedirectory);
  XtFree(cmapfnameppm);
  XtFree(cmapfnameascii);

}

void Xet::cmapfilenameTBL(){
  Widget filedirectoryTF  = nameToWidget("cmapfilenameTBL*filedirectoryTF");
  XmTextFieldSetString(filedirectoryTF,CURRENT_DIRECTORY);

}

void Xet::loadlocus(){
  Widget locusList         = nameToWidget("locusTBL*locusList");
  Widget filer             = nameToWidget("LoadlocusFSB*loadlocusfsb");
  Widget selectedText      = XmFileSelectionBoxGetChild(filer,XmDIALOG_TEXT);
  char *fileName = XmTextGetString(selectedText);
  Widget locusNameTF       = nameToWidget("locusTBL*locusNameTF");
  char *locusName       = XmTextFieldGetString(locusNameTF);
  Locus *l = locusPool->findLocus(locusName);
  if(l != NULL){
    popupModalInfoBox("locus <%s> already exist, try another name\n",locusName);
    XtFree(fileName);
    XtFree(locusName);
    return;
  }
  char buffer[60];
  Locus *locus = new Locus(10);
  if(locus->load(fileName) == 1){
    if(locus->getSize() == 0){
      insertStringtoInfo("Empty locus object, ignored\n");
      delete locus;
      XtFree(fileName);
      XtFree(locusName);
      return;
    }
    locus->setName(locusName);
    locusPool->addLocus(locusName,locus);
    sprintf(buffer,LOCUS_FORMAT,locusName,locus->getRefCounter(),
	    locus->getSize());
    PutTextInListWidget(locusList,buffer);
  }else
    popupModalInfoBox("loading locus file <%s> failed",fileName);

  XtFree(fileName);
  XtFree(locusName);
}

void Xet::writeLocus(){
  Widget writeLocusCnameTF = nameToWidget("writeLocusTBL*writeLocusCnameTF");
  Widget writeLocusDnameTF = nameToWidget("writeLocusTBL*writeLocusDnameTF");
  char *writeLocusCname = XmTextFieldGetString(writeLocusCnameTF);
  char *writeLocusDname = XmTextFieldGetString(writeLocusDnameTF);
  Locus *l = locusPool->findLocus(writeLocusCname);
  if(l){
    if(l->writeOut(writeLocusDname) == 0)
      popupModalInfoBox("can not open file <%s>",writeLocusDname);
  }else
    popupModalInfoBox("it is impossible, locus <%s> disappeared",writeLocusCname);
  insertStringtoInfo("Wrote locus to <%s>",writeLocusDname);
  XtFree(writeLocusCname);
  XtFree(writeLocusDname);
}

void Xet::prewriteLocus(){
  Widget locusList = nameToWidget("locusTBL*locusList");
  char *buf = whichItemInXmListIsSelected(locusList);
  if(buf){
    char itemno[10],locusid[40];
    sscanf(buf,LOCUS_FORMAT_SCAN_NAME, itemno, locusid);
    WcCreateRoot(Dpy,"writeLocusTBL");
    Widget writeLocusCnameTF = nameToWidget("writeLocusTBL*writeLocusCnameTF");
    XmTextFieldSetString(writeLocusCnameTF,locusid);
  }else{
    popupModalInfoBox("Must select an item\n");
    Widget w = nameToWidget("locusTBL*locusWrite");
    XtSetSensitive(w,True);
  }
}

void Xet::preschedulerWrite(){
  Widget schedList =  nameToWidget("schedulerTBL*schedList");
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],clipName[200],runmode[20];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, clipName,runmode);
    WcCreateRoot(Dpy,"writeSchedulerTBL");
    Widget writeCnameTF = nameToWidget("writeSchedulerTBL*writeSchedulerCnameTF");
    XmTextFieldSetString(writeCnameTF,clipName);
  }else{
    popupModalInfoBox("Must select an item\n");
    genericCancel("schedulerTBL*schedulerWrite");
  }
}

void Xet::writeScheduler(){
  Widget writeSchedulerCnameTF = nameToWidget("writeSchedulerTBL*writeSchedulerCnameTF");
  Widget writeSchedulerDnameTF = nameToWidget("writeSchedulerTBL*writeSchedulerDnameTF");
  char *writeSchedulerCname = XmTextFieldGetString(writeSchedulerCnameTF);
  char *writeSchedulerDname = XmTextFieldGetString(writeSchedulerDnameTF);
  if(scheduler->writeOut(writeSchedulerDname,writeSchedulerCname) == 0)
    popupModalInfoBox("can not open file <%s>",writeSchedulerDname);
  else
    insertStringtoInfo("Wrote scheduler to <%s>",writeSchedulerDname);
  XtFree(writeSchedulerCname);
  XtFree(writeSchedulerDname);
}
