//
//  Xet - X Entertainment (a tool to create animated images)
//
//
//  Author:   Ko-Haw Nieh & Coalition Software, (nieh@netcom.com)
//
//
//  Copyright 1992, 1993 by Ko-Haw Nieh & Coalition Software
//
// Permission to use, copy, and distribute for non-commercial purposes,
// is hereby granted without fee, providing that the above copyright
// notice appear in all copies and that both the copyright notice and this
// permission notice appear in supporting documentation. 
//
// This software is provided as-is and without any warranty of any kind.

#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include "config.h"
#include "xet.h"
#include <libc.h>
#include <osfcn.h>
#include <string.h>
#include "util.h"
#include <Xm/ToggleB.h>
#include <Xm/FileSB.h>
#include <X11/cursorfont.h>
#include "colorImage.h"
#include "wrapper.h"
#include "color.h"
#include "run_fly.h"
#include "run_glide.h"
#include "run_random.h"
#include "run_locus.h"
#include "run_fixed.h"
#include "run_blink.h"
#include "run_movie.h"
#include "procSpooler.h"

extern Color          *currentCmap;

extern int    asyncResult;

void Xet::schedulerMoveToMovie(){
  Widget schedList  =  nameToWidget("schedulerTBL*schedList");
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  char buffer[100];
  char *buf = whichItemInXmListIsSelected(schedList);
  if(buf){
    char itemno[10],clipName[256],runmode[10];
    sscanf(buf,SCHEDULER_FORMAT_SCAN_NAME, itemno, clipName,runmode);
    RunBase  *cat = scheduler->findRunObject(clipName);
    if(strcmp(runmode,"Movie") == 0){
      RunBase *ronew;
      int i;
      for(i=0;;i++){
	ronew = ((RunMovie *)cat)->getAnObject(i);
	if(ronew == NULL)
	  break;
	scheduler->addObjToMovie(ronew);
      }
      Widget movieNameTF = nameToWidget("schedulerTBL*movieNameTF");
      XmTextFieldSetString(movieNameTF,cat->name);
      updateMovieList();
    }else{
      char *tc = cat->getTermCondValueInString();
      sprintf(buffer,MOVIE_FORMAT,clipName,runmode,cat->interval,tc);
      PutTextInListWidget(movieList,buffer);
      scheduler->addObjToMovie(clipName);
    }
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}
void Xet::movieExtract(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  int which = whichLineInXmListIsSelected(movieList);
  if(which >= 0){
    scheduler->extractMovieObject(which);
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");

}
void Xet::updateMovieList(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  scheduler->updateMovieList(movieList);
}

void Xet::movieMoveUp(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  int which = whichLineInXmListIsSelected(movieList);
  if(which >= 0){
    scheduler->MoveMovieUp(which);
    updateMovieList();
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}

void Xet::movieMoveDown(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  int which = whichLineInXmListIsSelected(movieList);
  if(which >= 0){
    scheduler->MoveMovieDown(which);
    updateMovieList();
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}

void Xet::movieMoveToFirst(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  int which = whichLineInXmListIsSelected(movieList);
  if(which >= 0){
    scheduler->MoveMovieToFirst(which);
    updateMovieList();
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}

void Xet::movieMoveToLast(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  int which = whichLineInXmListIsSelected(movieList);
  if(which >= 0){
    scheduler->MoveMovieToLast(which);
    updateMovieList();
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}

void Xet::movieDelete(){
  Widget movieList  =  nameToWidget("schedulerTBL*movieList");
  int which = whichLineInXmListIsSelected(movieList);
  if(which >= 0){
    Widget areYouSureTBL = createAreYouSureDialog("Delete selected movie object, Are you sure ?");
    int rtn;
    rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(rtn){    
      scheduler->MovieDelete(which);
      updateMovieList();
    }
  }else
    popupModalInfoBox("!!! Must select an item !!!\n");
}

void Xet::movieDeleteAll(){
  Widget movieList     =  nameToWidget("schedulerTBL*movieList");
  if(movieList){
    Widget areYouSureTBL = createAreYouSureDialog("Delete all objects in current movie, Are you sure ?");
    int rtn;
    rtn = performAsyncInteraction(Dpy,areYouSureTBL);
    if(rtn){   
      scheduler->MovieDeleteAll();
      deleteAllItemsOfXmList(movieList);
      Widget movieNameTF = nameToWidget("schedulerTBL*movieNameTF");
      XmTextFieldSetString(movieNameTF,"");
    }
    destroyAreYouSureDialog();
  }
}

void Xet::movieCommit(){
  Widget movieList   =  nameToWidget("schedulerTBL*movieList");
  Widget movieNameTF = nameToWidget("schedulerTBL*movieNameTF");
  char  *movieName   = XmTextFieldGetString(movieNameTF);
  if(strcmp("",movieName) != 0){
    if(scheduler->MovieCommit(movieName) == 1){
      deleteAllItemsOfXmList(movieList);
      XmTextFieldSetString(movieNameTF,"");
    }else
      popupModalInfoBox("Animation object <%s> already exist or movie list is empty.",movieName);
  }else
    popupModalInfoBox("Must specify a movie name");
  XtFree(movieName);
}
